@@ -704,6 +704,8 @@ void WSCbaseDialog::onGuiPolicyChange(long policy){
       _ok_btn->redraw();
     }
   }
+  //to destroy internal pixmap..
+  _cmd_form->internalUpdate();
   WSCbase::onGuiPolicyChange(policy);
 }
 void WSCbaseDialog::onFocusChange(WSCbool fl){
