@@ -679,4 +679,6 @@ void WSCoption::onGuiPolicyChange(long policy){
     redraw();
   }
 }
-
+char* WSCoption::getDefaultProperty(){
+  return WSNvalue;
+}
