@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -61,6 +61,9 @@ WSCvbtn::WSCvbtn(WSCbase* base, char* objname):
   _emboss = True;
   _focus_border = True;
   _mask_images = NULL;
+  _ext_draw_proc = NULL;
+  _no_background = False;
+  WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCvbtn);
@@ -118,6 +121,8 @@ void WSCvbtn::setWorkWSNbacktab(char*) {}
 void WSCvbtn::getWorkWSNbacktab(char**) {}
 
 WSCvbtn::~WSCvbtn() {
+  WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
+
   if (getFocus() != False){
     WSGIappKeyboard()->resetFocusInputWorkProc();
   }
@@ -388,6 +393,54 @@ long WSCvbtn::draw(){
     return WS_ERR;
   }
 
+  if (WSGIappDev()->getExtGuiPolicySelected() != False){
+    if (_ext_draw_proc == NULL){
+      _ext_draw_proc = (void(*)(void*))WSGIappDev()->
+                               getExtGuiPolicyProc("WSCvbtn::draw");
+    }
+    if (_ext_draw_proc != NULL){
+      WSCvbtn_draw_struct st;
+      st._this = this;
+      st.dev = dev;
+      st.x = _x;
+      st.y = _y;
+      st.width = _w;
+      st.height = _h;
+      st.shadow_thickness = _shadow_thick;
+      st.blinking = _bl_fl;
+      st.blink_fore = WSGIappBlink()->getBlinkFore(_blink_rate);
+      st.shadow_type = _shadow_type;
+      st.blink_type  = _bl_type;
+      st.twin_blink   = _tw_fl;
+      st.fore_color   = _fore_color;
+      st.back_color   = _back_color;
+      st.blink_color  = _blink_color;
+      st.back_blink_color = _bg_blink_color;
+      st.top_shadow_color = _ts_color;
+      st.bottom_shadow_color = _bs_color;
+      st.label_string = _label_string;
+      st.label_pixmap = _label_pixmap;
+      st.blink_pixmap = _blink_pixmap;
+      st.push_pixmap = _push_pixmap;
+      st.press_state = _press_state;
+      st.orientation = _orientation;
+      st.gradation = _grad_no;
+      st.gradation_margin = _grad_margin;
+      st.margin_left = _margin_left;
+      st.margin_right = _margin_right;
+      st.margin_top = _margin_top;
+      st.margin_bottom = _margin_bottom;
+      st.focus_border = _focus_border;
+      st.emboss = _emboss;
+      st.font = _font;
+      st.reverse_flag = _rv_fl;
+      st.enable_focus_move = _ef_move;
+      st.no_background = _no_background;
+      _ext_draw_proc(&st);
+      return WS_NO_ERR;
+    }
+  }
+
   short x = _x;
   short y = _y;
   WSCushort w = _w;
@@ -802,3 +855,26 @@ void WSCvbtn::setDrawFocusBorder(WSCbool fl){
     _focus_border = True;
   }
 }
+
+void WSCvbtn::setNoBackground(WSCbool fl){
+  _no_background = fl;
+}
+void(*WSCvbtn::_ext_draw_proc)(void*) = NULL;
+WSCstring WSCvbtn::_policy_name;
+
+void WSCvbtn::onGuiPolicyChange(long policy){
+  WSCvlabel::onGuiPolicyChange(policy);
+  if (WSGIappDev()->getExtGuiPolicySelected() == False){
+    _ext_draw_proc = NULL;
+  }else{
+    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    }else{
+      _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCvbtn::draw");
+    }
+  }
+  if (getVisible() != False){
+    needUpdate();
+    setAbsoluteDraw(True);
+    redraw();
+  }
+}
