@@ -343,7 +343,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
   }
 
   long ialign = getAlignment();
-  WSCushort iww = w - st*2;
+  WSCushort iww = ww;
   if (_use_icon != 0){
     if (_vk_draw == WS_TREE){
       xx = hh + indent_level*hh;
