@@ -30,16 +30,30 @@
 WSMFclassInit(WSDfont,WSDres);
 
 WSDfont::WSDfont(){
+  _virtual_flag = False;
 }
 
 WSDfont::~WSDfont(){
 }
 
 long WSDfont::setFontName(char* cname){
-   return setSrc(cname);
+  if (_virtual_flag != False){
+    _virtual_name.setString(cname);
+    if (_value1 != -1){
+      destroy();
+      setValue1(-1);
+      setValue2(-1);
+      setValue3(-1);
+    }
+    return WS_NO_ERR;
+  }
+  return setSrc(cname);
 }
 char* WSDfont::getFontName(){
-   return getSrc();
+  if (_virtual_flag != False){
+    return _virtual_name.getString();
+  }
+  return getSrc();
 }
 long WSDfont::setPrm(long /*kind*/,long  /*val*/){
   return WS_NO_ERR;
@@ -70,3 +84,6 @@ long WSDfont::getStringHeight(char* s,long en){
 long WSDfont::getFontHeight(){
   return getStringHeight((WSCstring*)NULL);
 }
+void WSDfont::setVirtualFlag(WSCbool fl){
+  _virtual_flag = fl;
+}
