@@ -331,8 +331,18 @@ long WSCsform::draw(){
 
 }
 
+extern long _get_ext_children_num(WSClistData* children);
+
 void WSCsform::onMousePress(WSCpoint* pt){
   long num = _pos_list.getNum();
+  WSClistData children = getChildren();
+  long cnum = _get_ext_children_num(&children);
+  if (cnum -1 < num){
+    num = cnum -1;
+  }
+  if (num < 0){
+    num = 0;
+  }
   long i;
   long mx = pt->x;
   long my = pt->y;
@@ -459,6 +469,7 @@ void WSCsform::_move_adjust_work(void* ptr){
     _sep_window->setProperty(WSNtitleBar,WS_NO_MANAGE);
     _sep_window->setProperty(WSNshadowThickness,0);
     _sep_window->setProperty(WSNbackColor,"#606060");
+    _sep_window->setProperty(WSNindicatorOn,False);
   }
   if (_sep_window == NULL){
     _move_timer->stopTimer();
@@ -480,6 +491,8 @@ void WSCsform::_delay_adjust_work(void* ptr){
   sform->_clear_area();
   sform->_delay_adjust_flag = 0;
 }
+
+
 void WSCsform::onMouseMove(WSCpoint* pt){
   if (_pressed != False){
     long pos = 0;
@@ -496,6 +509,14 @@ void WSCsform::onMouseMove(WSCpoint* pt){
     long next = 0;
     long num =  _pos_list.getNum();
 
+    WSClistData children = getChildren();
+    long cnum = _get_ext_children_num(&children);
+    if (cnum -1 < num){
+      num = cnum -1;
+    }
+    if (num < 0){
+      num = 0;
+    }
     if (_hit_bar > 0){
       pre = (long)_pos_list[_hit_bar-1];
     }
@@ -555,6 +576,14 @@ void WSCsform::onMouseMove(WSCpoint* pt){
     WSDdev* dev = getowndev();
     if (dev != NULL){
       long num = _pos_list.getNum();
+      WSClistData children = getChildren();
+      long cnum = _get_ext_children_num(&children);
+      if (cnum -1 < num){
+        num = cnum -1;
+      }
+      if (num < 0){
+        num = 0;
+      }
       long i;
       long mx = pt->x;
       long my = pt->y;
@@ -647,6 +676,34 @@ void WSCsform::onResize(WSCrect* area){
 //  needUpdate();
 }
 
+long _get_ext_children_num(WSClistData* children){
+    long i=0;
+    long cnt=0;
+    WSCbase* child;
+    while(1){
+      child =(WSCbase*)(*children)[i];
+      if(child == NULL){
+        break;
+      }
+      if (child->cast("WSCgripHand") != NULL){
+        i++;
+      }else{
+        long nomanaged = (long)child->getUserData("sform-managed");
+        if (nomanaged != False){
+          i++;
+          continue;
+        }
+        WSCbool fl = child->getProperty(WSNvis);
+        if (fl == False){
+          i++;
+          continue;
+        }
+        cnt++;
+        i++;
+      }
+    }
+    return cnt;
+}
 WSCbase* _get_ext_child(WSClistData* children,long no){
     long i=0;
     long cnt=0;
@@ -656,7 +713,6 @@ WSCbase* _get_ext_child(WSClistData* children,long no){
       if(child == NULL){
         return NULL;
       }
-//      if (child->getInternalObject() != False){
       if (child->cast("WSCgripHand") != NULL){
         i++;
       }else{
@@ -665,6 +721,11 @@ WSCbase* _get_ext_child(WSClistData* children,long no){
           i++;
           continue;
         }
+        WSCbool fl = child->getProperty(WSNvis);
+        if (fl == False){
+          i++;
+          continue;
+        }
         if (no == cnt){
           return child;
         }
