@@ -132,7 +132,14 @@ WSDbtronAppDev::WSDbtronAppDev(){
   _have_color_map = False;
   _color_map = NULL;
 }
-WSDbtronAppDev::~WSDbtronAppDev(){}
+WSDbtronAppDev::~WSDbtronAppDev(){
+  if (_fgid != NULL){
+    delete _fgid_buf;
+    gcls_env(_fgid);
+    _fgid = NULL;
+    _fgid_buf = NULL;
+  }
+}
 
 WSCushort WSDbtronAppDev::getWidth(){
 //  return (WSCushort)DisplayWidth(display(),DefaultScreen(display()));
@@ -1175,7 +1182,9 @@ dbprintf("WSDbtronAppDev::setGid.. gid=0x%x\n",gid);
     }
   }
   _gid = gid;
-  _fgid = getNewMemGid(32,32);
+  if (_fgid == NULL){
+    _fgid = getNewMemGid(32,32,&_fgid_buf);
+  }
 }
 WSCbool WSDbtronAppDev::getGidInitialized(){
   if (_gid != 0){
@@ -1189,7 +1198,7 @@ W WSDbtronAppDev::getFontGid(){
 WSCbool WSDbtronAppDev::haveColorMap(){
   return _have_color_map;
 }
-GID WSDbtronAppDev::getNewMemGid(WSCulong w,WSCulong h){
+GID WSDbtronAppDev::getNewMemGid(WSCulong w,WSCulong h,UB** ptr){
   if (getGidInitialized() == False){
 //printf(">> WSDbtronAppDev::getNewMemGid not initialized.. \n");
     return 0;
@@ -1209,6 +1218,7 @@ GID WSDbtronAppDev::getNewMemGid(WSCulong w,WSCulong h){
   bmp.bounds.c.top = 0;
   bmp.bounds.c.bottom = h;
   bmp.baseaddr[0] = (UB*)malloc(bmp.rowbytes * h);
+  *ptr = bmp.baseaddr[0];
   return gopn_mem(NULL,&bmp,(B*)cspec);
 }
 
