@@ -322,7 +322,7 @@ long WSDbtronDraw::beginDraw(WSDdev* dev,long val1,long val2,long val3){
   _clip_x = dx;
   _clip_y = dy;
   _clip_w = dw;
-  _clip_h = dw;
+  _clip_h = dh;
 
 //printf("gset_vis4 %d,%d,%d,%d\n",r.c.left,r.c.top,r.c.right,r.c.bottom);
   gset_vis(_gid,r);
@@ -779,6 +779,7 @@ long WSDbtronDraw::drawPoly(WSCpoint* points,long num){
     poly->size = num;
   }
   gfra_pol(_gid,poly,(_line_type_val<<8)+_line_width,&pat,G_STORE);
+  delete poly;
   return WS_NO_ERR;
 }
 
@@ -818,6 +819,7 @@ long WSDbtronDraw::drawFillPoly(WSCpoint* points,long num){
     poly->size = num;
   }
   gfil_pol(_gid,poly,&pat,G_STORE);
+  delete poly;
   return WS_NO_ERR;
 }
 
@@ -891,18 +893,20 @@ dbprintf(">> WSDbtronDraw::drawImage mem_gid1=0x%x\n",mem_gid1);
   WSGFandArea(x,y,w,h,_clip_x-px,_clip_y-py,_clip_w,_clip_h,&dx1,&dy1,(WSCulong*)&dw1,(WSCulong*)&dh1);
   WSGFandArea(dx1,dy1,dw1,dh1,x+xoff,y+yoff,pw,ph,&dx,&dy,(WSCulong*)&dw,(WSCulong*)&dh);
   if ((long)mem_gid2 != -1 && mem_gid2 != 0){
+    RECT from;
+    from.c.left = 0;
+    from.c.top = 0;
+    from.c.right = pw;
+    from.c.bottom = ph;
+    RECT to;
+    to.c.left = 0;
+    to.c.top = 0;
+    to.c.right = pw;
+    to.c.bottom = ph;
     if (WSGIbtronAppDev()->haveColorMap() != False){
-      RECT from;
-      from.c.left = 0;
-      from.c.top = 0;
-      from.c.right = pw;
-      from.c.bottom = ph;
-      RECT to;
-      to.c.left = 0;
-      to.c.top = 0;
-      to.c.right = pw;
-      to.c.bottom = ph;
       gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_AND);
+    }else{
+      gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_ANDN);
     }
   }
   RECT from;
@@ -966,18 +970,20 @@ dbprintf(">> WSDbtronDraw::drawStretchedImage mem_gid=0x%x\n",mem_gid1);
     py = 0;
   }
   if ((long)mem_gid2 != -1 && mem_gid2 != 0){
+    RECT from;
+    from.c.left = 0;
+    from.c.top = 0;
+    from.c.right = iw;
+    from.c.bottom = ih;
+    RECT to;
+    to.c.left = 0;
+    to.c.top = 0;
+    to.c.right = iw;
+    to.c.bottom = ih;
     if (WSGIbtronAppDev()->haveColorMap() != False){
-      RECT from;
-      from.c.left = 0;
-      from.c.top = 0;
-      from.c.right = iw;
-      from.c.bottom = ih;
-      RECT to;
-      to.c.left = 0;
-      to.c.top = 0;
-      to.c.right = iw;
-      to.c.bottom = ih;
       gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_AND);
+    }else{
+      gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_ANDN);
     }
   }
   RECT from;
@@ -1038,20 +1044,22 @@ dbprintf(">> WSDbtronDraw::drawImage mem_gid=0x%x\n",mem_gid1);
   long dw,dh;
   WSGFandArea(x,y,w,h,_clip_x-px,_clip_y-py,_clip_w,_clip_h,&dx,&dy,(WSCulong*)&dw,(WSCulong*)&dh);
   if ((long)mem_gid2 != -1 && mem_gid2 != 0){
+    WSCushort pw = image->getImageWidth();
+    WSCushort ph = image->getImageHeight();
+    RECT from;
+    from.c.left = 0;
+    from.c.top = 0;
+    from.c.right = pw;
+    from.c.bottom = ph;
+    RECT to;
+    to.c.left = 0;
+    to.c.top = 0;
+    to.c.right = pw;
+    to.c.bottom = ph;
     if (WSGIbtronAppDev()->haveColorMap() != False){
-      WSCushort pw = image->getImageWidth();
-      WSCushort ph = image->getImageHeight();
-      RECT from;
-      from.c.left = 0;
-      from.c.top = 0;
-      from.c.right = pw;
-      from.c.bottom = ph;
-      RECT to;
-      to.c.left = 0;
-      to.c.top = 0;
-      to.c.right = pw;
-      to.c.bottom = ph;
       gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_AND);
+    }else{
+      gcop_bmp(mem_gid2,&from,mem_gid1,&to,NULL,G_ANDN);
     }
   }
  
@@ -1327,6 +1335,7 @@ static WSDcolor* color = NULL;
   }
 
 
+  long line_type_val = LN_SOLID;
 
   col1->getRGB(&r1,&g1,&b1);
   col2->getRGB(&r2,&g2,&b2);
@@ -1412,7 +1421,7 @@ static WSDcolor* color = NULL;
       p1.y = y +dy;
       p2.x = x +dx;
       p2.y = y1+dy;
-      gdra_lin(_gid,p1,p2,(_line_type_val<<8)+wval,&pat,G_STORE);
+      gdra_lin(_gid,p1,p2,(line_type_val<<8)+wval,&pat,G_STORE);
     }
   }else
   if (type == WS_GR_LB_RT || type == WS_GR_RT_LB){
@@ -1484,7 +1493,7 @@ static WSDcolor* color = NULL;
       p1.y = y1+dy;
       p2.x = x1+dx;
       p2.y = y +h+dy;
-      gdra_lin(_gid,p1,p2,(_line_type_val<<8)+wval,&pat,G_STORE);
+      gdra_lin(_gid,p1,p2,(line_type_val<<8)+wval,&pat,G_STORE);
     }
   }else
   if (type == WS_GR_T_B || type == WS_GR_B_T){
@@ -1554,7 +1563,7 @@ static WSDcolor* color = NULL;
       p1.y = y+h1 +dy;
       p2.x = x+w +dx;
       p2.y = y+h1+dy;
-      gdra_lin(_gid,p1,p2,(_line_type_val<<8)+wval,&pat,G_STORE);
+      gdra_lin(_gid,p1,p2,(line_type_val<<8)+wval,&pat,G_STORE);
     }
   }else{
     long max = w* (100 -gradm)/100;
@@ -1623,7 +1632,7 @@ static WSDcolor* color = NULL;
       p1.y = y +dy;
       p2.x = x+w1 +dx;
       p2.y = y+h+dy;
-      gdra_lin(_gid,p1,p2,(_line_type_val<<8)+wval,&pat,G_STORE);
+      gdra_lin(_gid,p1,p2,(line_type_val<<8)+wval,&pat,G_STORE);
     }
   }
   long linet = _line_type;
