@@ -80,6 +80,7 @@ WSDbtronFormDev::WSDbtronFormDev(){
   _window = 0;
   _gid = 0;
   _mem_gid = 0;
+  _mem_gid_buf = NULL;
   _expose_sequence = False;
   _clip_num = 0;
 //  _pwidget = 0;
@@ -694,6 +695,9 @@ long WSDbtronFormDev::createPixmap(){
   return WS_NO_ERR;
 }
 void WSDbtronFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCushort h){
+  if (getParentVisible() == False){
+    return;
+  }
   if (w <1){
     w = 1;
   }
@@ -704,8 +708,11 @@ void WSDbtronFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCusho
   if (getParentMemGID() == 0){
     return;
   }
-  _setup_pixmap(x,x,w,h,_bg,_bg_pix);
+//printf("recursiveExposeExecute %s xywh=%d,%d,%d,%d\n",getAttachedClient()->getInstanceName(),x,y,w,h);
+  _setup_pixmap(x,y,w,h,_bg,_bg_pix);
+
   exposeExecute(x,y,w,h);
+
   short px,py;
   getAbstPos(&px,&py);
 #ifdef F_MEMGID
@@ -724,7 +731,7 @@ void WSDbtronFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCusho
       long dx,dy;
       WSCulong dw,dh;
       WSGFandArea(cx,cy,cw,ch,x+px,y+py,w,h,&dx,&dy,&dw,&dh);
-      child->recursiveExposeExecute(dx -cx, dy -cy, dw, dh);
+      child->recursiveExposeExecute(dx -cx, dy -cy, dw+1, dh+1);
     }
   }
 #endif
@@ -813,7 +820,10 @@ void WSDbtronFormDev::redrawAll(){
 //      pdev->_setup_pixmap(dx - pdev->_w_x, dy - pdev->_w_y,
 //                        dw, dh, pdev->_bg,pdev->_bg_pix);
 //    pdev->_setup_pixmap(dx,dy,dw,dh,pdev->_bg,pdev->_bg_pix);
-    pdev->recursiveExposeExecute(dx,dy,dw,dh);
+//printf("redrawAll %s xywh=%d,%d,%d,%d\n",getAttachedClient()->getInstanceName(),dx,dy,dw,dh);
+
+    pdev->recursiveExposeExecute(dx,dy,dw+1,dh+1);
+
     RECT r;
     r.c.left = dx;
     r.c.right = dx + dw;
@@ -870,6 +880,12 @@ dbprintf("WSDbtronFormDev::setVisible fl=%d\n",fl);
 
     if (bk2 != fl){
       redrawAll();
+#if 0 
+      WSDbtronFormDev* pdev = getMostParentAreaDev();
+      if (pdev != NULL && pdev->getVisible() != False){
+        pdev->addUpdateList();
+      }
+#endif
     }
 #if 0
     WSDbtronFormDev* pdev = getMostParentAreaDev();
@@ -902,16 +918,26 @@ dbprintf("WSDbtronFormDev::setVisible fl=%d\n",fl);
     }
 
   }else{
-    if (_window == 0){
+    if (bk2 != fl){
        createWindow();
     }
     if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
 //printf("WSDbtronFormDev::setVisible fl=%d %s\n",fl,getAttachedClient()->getInstanceName());
 
-      createPixmap();
+      if (bk2 != fl){
+        createPixmap();
 //      if (WSGIbtronAppDev()->getMouseGrabbed() != False){
+#if 1 //TEST
+//        WSDbtronFormDev* pdev = getMostParentAreaDev();
+//        if (pdev != NULL && pdev->getVisible() != False){
+//          pdev->addUpdateList();
+//        }
+        redrawAll();
+#else
         _copy_area(0,0,_w_w,_w_h,0,0,WS_PIXMAP_TO_WINDOW);
+#endif
 //      }
+      }
     }
   }
   return WS_NO_ERR;
@@ -1519,6 +1545,7 @@ extern W _fep_popup;
   memset(winlist,0,sizeof(winlist));
   wget_inf(WI_WINLIST,winlist,sizeof(winlist));
   for(i=0; i<N_VIEWR; i++){
+//printf("wget_inf i=%d win=0x%x\n",i,winlist[i]);
     if (N_VIEWR < cnt){
       break;
     }
@@ -1547,26 +1574,37 @@ extern W _fep_popup;
     cnt++;
   }
 
-  if (cnt > 0){
-    gset_for(getParentGID(),rlist);
-  }else{
-    gset_for(getParentGID(),NULL);
-  }
+
+//  if (cnt > 0){
+//    gset_for(getParentGID(),rlist);
+//  }else{
+//    gset_for(getParentGID(),NULL);
+//  }
 
 
-  if (num == 0){
+//  if (num == 0){
 //dbprintf("WSDbtronFormDev::setDefaultRegion NULL\n");
-    gset_for(getContextResource(),NULL);
-    return;
+//    gset_for(getContextResource(),NULL);
+//    return;
+//  }
+
+  WSDbtronFormDev* pdev = getParentAreaDev();
+  WSClistData bdlist;
+  if (pdev != NULL){
+    WSClistData bdlist2 = pdev->getChildDevList();
+    bdlist = bdlist2;
   }
+  long bnum = bdlist.getNum();
 
   long next=0;
   short dx,dy;
   WSCushort dw,dh;
   getAbstSize(&dx,&dy,&dw,&dh);
 
-  RLIST* rc = new RLIST[num+1];
+  RLIST* rc = new RLIST[num+bnum+1];
+  RLIST* rc_mem = new RLIST[num+bnum+1];
   rc[0].next = NULL;
+  rc_mem[0].next = NULL;
   for(i=0; i<num; i++){
     WSDbtronFormDev* dev = (WSDbtronFormDev*)dlist[i];
 //printf("setDefaultRegion %d %s vis=%d\n",i,dev->getAttachedClient()->getInstanceName(),dev->getParentVisible());
@@ -1581,6 +1619,7 @@ extern W _fep_popup;
     PNT pt;
     dev->getAbstSize(&cx,&cy,&cw,&ch);
     rc[next].next = NULL;
+    rc_mem[next].next = NULL;
     if (cw <1){
       continue;
     }
@@ -1590,26 +1629,119 @@ extern W _fep_popup;
 //dbprintf("WSDbtronFormDev::setDefaultRegion child->%s %d,%d,%d,%d\n",dev->getAttachedClient()->getInstanceName(),cx,cy,cw,ch);
     pt.x = cx;
     pt.y = cy;
-    gcnv_abs((W)getContextResource(),&pt);
+    gcnv_abs((W)getParentGID(),&pt);
+
     rc[next].rcomp.c.left = pt.x;
     rc[next].rcomp.c.top = pt.y;
     rc[next].rcomp.c.right = pt.x + cw;
     rc[next].rcomp.c.bottom = pt.y + ch;
+//printf("WSDbtronFormDev::setDefaultRegion %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
+
+    if (getParentMemGID() != 0){
+      pt.x = cx;
+      pt.y = cy;
+      rc_mem[next].rcomp.c.left = pt.x;
+      rc_mem[next].rcomp.c.top = pt.y;
+      rc_mem[next].rcomp.c.right = pt.x + cw;
+      rc_mem[next].rcomp.c.bottom = pt.y + ch;
+    }
+
+//printf("WSDbtronFormDev::setDefaultRegion2 %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
 //dbprintf("WSDbtronFormDev::setDefaultRegion %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
-    if (i == 0){
-    }else{
+    if (next >  0){
       rc[next-1].next = &rc[next];
+      rc_mem[next-1].next = &rc_mem[next];
     }
     next++;
   }
+
+  WSCbool found_self = False;
+  for(i=0; i<bnum; i++){
+    WSDbtronFormDev* dev = (WSDbtronFormDev*)bdlist[i];
+//printf("setDefaultRegion %d %s vis=%d\n",i,dev->getAttachedClient()->getInstanceName(),dev->getParentVisible());
+    if (WSDbtronFormDev::isValidInstance(dev) == False){
+      continue;
+    }
+    if (dev->getParentVisible() == False){
+      continue;
+    }
+    if (dev == this){
+      found_self = True;
+      continue;
+    }
+    if (found_self == False){
+      continue;
+    }
+    short cx,cy;
+    WSCushort cw,ch;
+    PNT pt;
+    dev->getAbstSize(&cx,&cy,&cw,&ch);
+    rc[next].next = NULL;
+    rc_mem[next].next = NULL;
+    if (cw <1){
+      continue;
+    }
+    if (ch <1){
+      continue;
+    }
+//dbprintf("WSDbtronFormDev::setDefaultRegion child->%s %d,%d,%d,%d\n",dev->getAttachedClient()->getInstanceName(),cx,cy,cw,ch);
+    pt.x = cx;
+    pt.y = cy;
+//    gcnv_abs((W)getContextResource(),&pt);
+    gcnv_abs((W)getParentGID(),&pt);
+
+    rc[next].rcomp.c.left = pt.x;
+    rc[next].rcomp.c.top = pt.y;
+    rc[next].rcomp.c.right = pt.x + cw;
+    rc[next].rcomp.c.bottom = pt.y + ch;
+
+    if (getParentMemGID() != 0){
+      pt.x = cx;
+      pt.y = cy;
+      rc_mem[next].rcomp.c.left = pt.x;
+      rc_mem[next].rcomp.c.top = pt.y;
+      rc_mem[next].rcomp.c.right = pt.x + cw;
+      rc_mem[next].rcomp.c.bottom = pt.y + ch;
+    }
+
+//dbprintf("WSDbtronFormDev::setDefaultRegion %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
+    if (next >  0){
+      rc[next-1].next = &rc[next];
+      rc_mem[next-1].next = &rc_mem[next];
+    }
+    next++;
+  }
+//printf("cnt=%d\n",cnt);
   if (next == 0){
 //dbprintf("WSDbtronFormDev::setDefaultRegion set_for=0x%x\n",getContextResource());
-    gset_for(getContextResource(),NULL);
+    if (getParentMemGID() != 0){
+      gset_for(getParentMemGID(),NULL);
+    }
+    if (getParentGID() != 0){
+      if (cnt > 0){
+//printf("clip cnt=%d\n",cnt);
+        gset_for(getParentGID(),rlist);
+      }else{
+        gset_for(getParentGID(),NULL);
+      }
+    }
+//    gset_for(getContextResource(),NULL);
   }else{
 //dbprintf("WSDbtronFormDev::setDefaultRegion set_for=0x%x\n",getContextResource());
-    gset_for(getContextResource(),rc);
+    if (getParentMemGID() != 0){
+      gset_for(getParentMemGID(),rc_mem);
+    }
+    if (getParentGID() != 0){
+//printf("add cnt=%d\n",cnt);
+      if (cnt > 0){
+        rc[next-1].next = &rlist[0];
+      }
+      gset_for(getParentGID(),rc);
+    }
+//    gset_for(getContextResource(),rc);
   }
   delete[] rc;
+  delete[] rc_mem;
 }
 void WSDbtronFormDev::drawBackGround(long x,long y,WSCulong w,WSCulong h){
   if (getParentVisible() == False){
@@ -1707,7 +1839,6 @@ void WSDbtronFormDev::drawBackGround(long x,long y,WSCulong w,WSCulong h){
       }
     }
   }
-
 }
 W WSDbtronFormDev::getParentWindow(){
   if (_window != NULL){
@@ -1879,8 +2010,9 @@ void WSDbtronFormDev::executeUpdate(){
     if (ret == 0){
       if (dev != NULL &&
         WSDbtronFormDev::isValidInstance(dev) != False){
+#if 1 //TEST
         dev->redrawAll();
-#if 0
+#else
           short dx,dy;
           WSCushort dw,dh;
           dev->getAbstSize(&dx,&dy,&dw,&dh);
