@@ -48,12 +48,16 @@ extern "C" {
 #include <btron/WSDbtronAppDev.h>
 #include <btron/WSDbtronColor.h>
 
-long WSGFbtronBufToPixmap(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes);
-long WSGFbtronBufToPixmapG(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes);
-long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h);
-long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,long* w,long* h);
-long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h);
-extern long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h);
+extern char* WSGFbtronAllocNewMemory(long);
+extern void WSGFbtronDeleteMemory(char*);
+
+long WSGFbtronBufToPixmap(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes,UB**,WSCbool reuse_mem);
+long WSGFbtronBufToPixmap2(unsigned char *buf,W* pixmap,W*,long width,long height,long* vw,long* vh,long row_bytes,UB**,UB**,WSCbool reuse_mem);
+long WSGFbtronBufToPixmapG(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes,UB**,WSCbool reuse_mem);
+long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,W*,long* w,long* h,UB**,UB**);
+long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,W*,long* w,long* h,UB**,UB**);
+long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h,UB**,UB**);
+extern long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h,UB**,UB**,WSCbool reuse_mem);
 
 
 WSMFclassInit(WSDbtronImage,WSDimage);
@@ -68,6 +72,8 @@ void  _btronImage_init(){
 };
 
 WSDbtronImage::WSDbtronImage(){
+  _pix_buf = NULL;
+  _pix_buf2 = NULL;
 }
 
 WSDbtronImage::~WSDbtronImage(){
@@ -109,8 +115,14 @@ dbprintf("IMG>> WSDbtronImage::initialize img=%s\n",in.getString());
      long width = _w;
      long height = _h;
      char* load_file_name = in.getString();
-     long reti = JpegReadFileToPixmap( load_file_name, &mem_gid,&mem_gid2,
-                                       &width,&height);
+     long reti = 0;
+     if (_mode & WS_IMAGE_MODE_REUSE_MEM){
+       reti = JpegReadFileToPixmap( load_file_name, &mem_gid,&mem_gid2,
+                                  &width,&height,&_pix_buf,&_pix_buf2,True);
+     }else{
+       reti = JpegReadFileToPixmap( load_file_name, &mem_gid,&mem_gid2,
+                                  &width,&height,&_pix_buf,&_pix_buf2,False);
+     }
 
      if (mem_gid == 0){
        mem_gid = -1;
@@ -133,7 +145,7 @@ dbprintf("IMG>> WSDbtronImage::initialize img=%s\n",in.getString());
      char* load_file_name = in.getString();
 dbprintf("IMG>> WSDbtronImage::initialize png..\n");
      long reti = PngReadFileToPixmap( load_file_name, &mem_gid,&mem_gid2,
-                                      &width,&height);
+                                      &width,&height,&_pix_buf,&_pix_buf2);
 dbprintf("IMG>> WSDbtronImage::initialize png done..\n");
      if (mem_gid == 0){
        mem_gid = -1;
@@ -151,6 +163,7 @@ dbprintf("IMG>> WSDbtronImage::initialize png done..\n");
 //WSMFtrace("WSDbtronImage::initialize %s\n",in.getString());
   if (in.isExist(".bmp") != -1 || in.isExist(".BMP") != -1){
      W mem_gid;
+     W mem_gid2;
      long width = _w;
      long height = _h;
      char* load_file_name = in.getString();
@@ -159,7 +172,7 @@ dbprintf("IMG>> WSDbtronImage::initialize png done..\n");
 //     GC gc = WSGIxwinAppDev()->appGC2();
 
      long reti = WSGFbtronBmpReadFileToPixmap(
-                       load_file_name, &mem_gid, &width,&height);
+                       load_file_name, &mem_gid,&mem_gid2, &width,&height,&_pix_buf,&_pix_buf2);
      if (mem_gid == 0){
        mem_gid = -1;
      }
@@ -167,7 +180,7 @@ dbprintf("IMG>> WSDbtronImage::initialize png done..\n");
        setImageWidth(width);
        setImageHeight(height);
        setValue1((long)mem_gid);
-       setValue2((long)0);
+       setValue2((long)mem_gid2);
      }else{
 //WSMFtrace("ERROR. WSDbtronImage::initialize file open error. file=%s\n",load_file_name);
      }
@@ -180,11 +193,19 @@ dbprintf("IMG>> WSDbtronImage::initialize done.\n",in.getString());
 long WSDbtronImage::destroy(){
   W memgid1 = (W)_value1;
   if (memgid1 != 0 && memgid1 != -1 ){
+    if (_mode & WS_IMAGE_MODE_REUSE_MEM){
+      WSGFbtronDeleteMemory((char*)_pix_buf);
+    }else{
+      delete _pix_buf;
+    }
+    _pix_buf = NULL;
     gcls_env(memgid1);
     setValue1((long)-1);
   }
   W memgid2 = (W)_value2;
   if (memgid2 != 0 && memgid2 != -1 ){
+    delete _pix_buf2;
+    _pix_buf2 = NULL;
     gcls_env(memgid2);
     setValue2((long)-1);
   }
@@ -238,7 +259,7 @@ int WSGFbtronreadLittleLong(FILE *fp, long *d) {
   return 0;
 }
 
-long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
+long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,W* pixmap2,long* w,long* h,UB** ptr,UB** ptr2){
 
   // bmp (Windows 3.x) header
   //       implemented
@@ -269,7 +290,9 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
   }
   if (long_tmp != 40){
     // not a Windows 3.x bmp file
-    return -1;
+    if (long_tmp != 0x6c){
+      return -1;
+    }
   }
   if (WSGFbtronreadLittleLong(ifile, &width) == -1){                 // 18
     return -1;
@@ -287,7 +310,8 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
   if (WSGFbtronreadLittleShort(ifile, &bit_cnt) == -1){              // 28
     return -1;
   }
-  if ((bit_cnt != 1) && (bit_cnt != 4) && (bit_cnt != 8) && (bit_cnt != 24)){
+  if ((bit_cnt != 1) && (bit_cnt != 4) && (bit_cnt != 8) &&
+      (bit_cnt != 24) && (bit_cnt != 32)){
     // bit_cnt=16 is not supported
     return -1;
   }
@@ -295,8 +319,12 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
     return -1;
   }
   if (cmptype > 2){
-    // compressed file is not supported
-    return -1;
+    if (cmptype == 3 && bit_cnt == 32){
+      // 24bit + mask.
+    }else{
+      // compressed file is not supported
+      return -1;
+    }
   }
   if (WSGFbtronreadLittleLong(ifile, &long_tmp) == -1){              // 34
     return -1;
@@ -313,6 +341,9 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
   if (WSGFbtronreadLittleLong(ifile, &long_tmp) == -1){              // 50
     return -1;
   }
+  if ((cmapsize == 0) && (bit_cnt <=8)){
+    cmapsize = 1 << bit_cnt;
+  }
   int n, c;
   for (n = 0; n < cmapsize; n++) {
     if ((c = WSGFbtrongetbyte(ifile)) == -1){
@@ -340,7 +371,12 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
     return -1;
   }
   unsigned char* buf = new unsigned char[width * height * 3];
+  unsigned char* buf_mask = NULL;
+  if (cmptype == 3){
+    buf_mask = new unsigned char[width * height*3];
+  }
   unsigned char *tmp, *ftmp;
+  unsigned char *tmp_mask;
 
   int line, dat, x, y;
 
@@ -505,30 +541,67 @@ long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h){
         break;
       case 24:
         tmp = buf;
+        tmp_mask = buf_mask;
         for (y = 0; y < height; y++){
-          ftmp = fbuf + ((height - 1 - y) * width * 3);
+          if (cmptype != 3){
+//            ftmp = fbuf + (height - 1 - y) * width * 3;
+            ftmp = fbuf + (height - 1 - y) * (((width * 3)+3)/4)*4;
+          }else{
+            ftmp = fbuf + ((height - 1 - y) * width * 4);
+          }
           for (x = 0; x < width; x++){
             *(tmp++) = *(ftmp + 2);       // red
             *(tmp++) = *(ftmp + 1);       // green
             *(tmp++) = *(ftmp + 0);       // blue
-            ftmp += 3;
+            if (cmptype != 3){
+              ftmp += 3;
+            }else{
+              *(tmp_mask++) = *(ftmp + 3);
+              ftmp += 4;
+            }
+          }
+        }
+        break;
+      case 32:
+        tmp = buf;
+        tmp_mask = buf_mask;
+        for (y = 0; y < height; y++){
+          ftmp = fbuf + ((height - 1 - y) * width * 4);
+          for (x = 0; x < width; x++){
+            *(tmp++) = *(ftmp + 2);       // red
+            *(tmp++) = *(ftmp + 1);       // green
+            *(tmp++) = *(ftmp + 0);       // blue
+
+            if (cmptype == 3){
+              *(tmp_mask++) = *(ftmp + 3);       // blue
+//printf("i=%d j=%d pt=%d mask=0x%x\n",y,x,tmp_mask-1 - buf_mask,*(tmp_mask -1));
+            }
+            ftmp += 4;
           }
         }
         break;
     }
   }
   delete fbuf;
-  long reti = WSGFbtronBufToPixmap(buf, pixmap, width, height,w,h,width*3);
+//  long reti = WSGFbtronBufToPixmap(buf, pixmap, width, height,w,h,width*3,ptr,False);
+  long reti =0;
+  if (cmptype == 3){
+    reti = WSGFbtronBufToPixmap2(buf, pixmap, pixmap2,width, height,w,h,width*3,ptr,ptr2,False);
+    delete buf;
+    delete buf_mask;
+  }else{
+    reti = WSGFbtronBufToPixmap(buf, pixmap, width, height,w,h,width*3,ptr,False);
+    delete buf;
+  }
 
-  delete buf;
 //  *w = width;
 //  *h = height;
   return reti;
 }
 
-long WSGFbtronBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight,long* w,long* h,long row_bytes){
+long WSGFbtronBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight,long* w,long* h,long row_bytes,UB** _gid_buf,WSCbool reuse_mem){
 
-  long stretched = 0;
+  long stretched = False;
   long vwidth = *w;
   long vheight = *h;
   if (vwidth == 0 || vheight == 0){
@@ -540,46 +613,239 @@ long WSGFbtronBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
     }else{
       vheight = (long)((double)vwidth * rheight/rwidth);
     }
-    stretched = 1;
+    if (vwidth == rwidth && vheight == rheight){
+      stretched = False;
+    }else{
+      stretched = True;
+    }
   }
   *w = vwidth;
   *h = vheight;
-
-  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight);
-//printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
-  if (*mem_gid == 0){
-    return -1;
-  }
   long i,j;
-  long ptr=0;
-  unsigned char r,g,b;
-  for(i=0; i<vheight; i++){
-    for(j=0; j<vwidth; j++){
-      if (stretched != 0){
-        ptr = (long)((double)i*rheight/vheight)*rwidth
-             + (long)((double)j*rwidth/vwidth);
-        ptr *= 3;
-      }
-      r = buf[ptr++];
-      g = buf[ptr++];
-      b = buf[ptr++];
-      unsigned long val = 0x10000000;
-      val |= r << 16;
-      val |= g << 8;
-      val |= b;
-      PIXVAL pixval;
-      gcnv_col(*mem_gid,val,&pixval);
-      PNT pt;
-      pt.x = j;
-      pt.y = i;
-      gdra_pnt(*mem_gid,pt,pixval,G_STORE);
+  UB* buf2 = NULL;
+  if (reuse_mem){
+    buf2 = (UB*)WSGFbtronAllocNewMemory( rheight * rwidth * 2);
+  }else{
+    buf2 = new UB [rheight * rwidth * 2];
+  }
+
+  long cnt = 0;
+  long ptr = 0;
+  DEV_SPEC* dspec = WSGIbtronAppDev()->getDevSpec();
+  WSCbool duplicate = False;
+  if ((dspec->attr & 0xf) != 1){
+    duplicate = True;
+  }else{
+    if (dspec->color[0] == 0x0b05 &&
+        dspec->color[1] == 0x0506 &&
+        dspec->color[2] == 0x0005){
+      //OK
+    }else{
+      duplicate = True;
     }
+  }
+
+  for(i=0; i<vheight; i++){
     ptr = row_bytes*i;
+    for(j=0; j<vwidth; j++){
+      UB r = buf[ptr++];
+      UB g = buf[ptr++];
+      UB b = buf[ptr++];
+      buf2[cnt++] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
+      buf2[cnt++] = (r & 0xf8) | ((g & 0xe0) >> 5);
+    }
+  }
+
+  BMP bmp;
+  bmp.planes = 1;
+  bmp.pixbits = 0x1010;
+  bmp.rowbytes = rwidth*2;
+  bmp.bounds.c.left = 0;
+  bmp.bounds.c.top = 0;
+  bmp.bounds.c.right = rwidth;
+  bmp.bounds.c.bottom = rheight;
+  bmp.baseaddr[0] = buf2;
+
+  CSPEC   csp;
+  csp.attr = DA_COLOR_RGB;
+//  csp.info[0] = 0x1008;
+//  csp.info[1] = 0x0808;
+//  csp.info[2] = 0x0008;
+//  csp.info[3] = 0;
+  csp.info[0] = 0x0b05;
+  csp.info[1] = 0x0506;
+  csp.info[2] = 0x0005;
+  csp.info[3] = 0;
+  csp.colmap = NULL;
+
+  W bgid = gopn_mem(NULL,&bmp,(B*)(&csp));
+//printf("bgid=0x%x\n",bgid);
+  if (duplicate == False && stretched == False){
+    *mem_gid = bgid;
+    *_gid_buf = buf2;
+  }else{
+    RECT r;
+    RECT r2;
+    r.c.left = 0;
+    r.c.top = 0;
+    r.c.right = rwidth;
+    r.c.bottom = rheight;
+    r2.c.left = 0;
+    r2.c.top = 0;
+    r2.c.right = vwidth;
+    r2.c.bottom = vheight;
+    if (stretched != 0){
+      r2.c.left = 0;
+      r2.c.top = 0;
+      r2.c.right = rwidth;
+      r2.c.bottom = rheight;
+    }
+
+    *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf);
+//printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
+    if (*mem_gid == 0){
+      if (reuse_mem){
+        WSGFbtronDeleteMemory((char*)buf2);
+      }else{
+        delete buf2;
+      }
+      return -1;
+    }
+    W ret = gcop_bmp(bgid,&r2,*mem_gid,&r,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+//  W ret = grst_bmp(*mem_gid,&r,&bmp,&r2,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+
+    if (reuse_mem){
+      WSGFbtronDeleteMemory((char*)buf2);
+    }else{
+      delete buf2;
+    }
+    gcls_env(bgid);
   }
   return 0;
 }
-long WSGFbtronBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheight,long* w,long* h,long row_bytes){
+long WSGFbtronBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheight,long* w,long* h,long row_bytes,UB** _gid_buf,WSCbool reuse_mem){
+  long stretched = False;
+  long vwidth = *w;
+  long vheight = *h;
+  if (vwidth == 0 || vheight == 0){
+    vwidth = rwidth;
+    vheight = rheight;
+  }else{
+    if ((long)((double)rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)((double)vheight * rwidth/rheight);
+    }else{
+      vheight = (long)((double)vwidth * rheight/rwidth);
+    }
+    if (vwidth == rwidth && vheight == rheight){
+      stretched = False;
+    }else{
+      stretched = True;
+    }
+  }
+  *w = vwidth;
+  *h = vheight;
+  long i,j;
+  UB* buf2 = NULL;
+  if (reuse_mem){
+    buf2 = (UB*)WSGFbtronAllocNewMemory( rheight * rwidth * 2);
+  }else{
+    buf2 = new UB [rheight * rwidth * 2];
+  }
 
+  long cnt = 0;
+  long ptr = 0;
+  DEV_SPEC* dspec = WSGIbtronAppDev()->getDevSpec();
+  WSCbool duplicate = False;
+  if ((dspec->attr & 0xf) != 1){
+    duplicate = True;
+  }else{
+    if (dspec->color[0] == 0x0b05 &&
+        dspec->color[1] == 0x0506 &&
+        dspec->color[2] == 0x0005){
+      //OK
+    }else{
+      duplicate = True;
+    }
+  }
+
+  for(i=0; i<vheight; i++){
+    ptr = row_bytes*i;
+    for(j=0; j<vwidth; j++){
+      UB r = buf[ptr];
+      UB g = buf[ptr];
+      UB b = buf[ptr++];
+      buf2[cnt++] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
+      buf2[cnt++] = (r & 0xf8) | ((g & 0xe0) >> 5);
+    }
+  }
+
+  BMP bmp;
+  bmp.planes = 1;
+  bmp.pixbits = 0x1010;
+  bmp.rowbytes = rwidth*2;
+  bmp.bounds.c.left = 0;
+  bmp.bounds.c.top = 0;
+  bmp.bounds.c.right = rwidth;
+  bmp.bounds.c.bottom = rheight;
+  bmp.baseaddr[0] = buf2;
+
+  CSPEC   csp;
+  csp.attr = DA_COLOR_RGB;
+//  csp.info[0] = 0x1008;
+//  csp.info[1] = 0x0808;
+//  csp.info[2] = 0x0008;
+//  csp.info[3] = 0;
+  csp.info[0] = 0x0b05;
+  csp.info[1] = 0x0506;
+  csp.info[2] = 0x0005;
+  csp.info[3] = 0;
+  csp.colmap = NULL;
+
+  W bgid = gopn_mem(NULL,&bmp,(B*)(&csp));
+//printf("bgid=0x%x\n",bgid);
+  if (duplicate == False && stretched == False){
+    *mem_gid = bgid;
+    *_gid_buf = buf2;
+  }else{
+    RECT r;
+    RECT r2;
+    r.c.left = 0;
+    r.c.top = 0;
+    r.c.right = rwidth;
+    r.c.bottom = rheight;
+    r2.c.left = 0;
+    r2.c.top = 0;
+    r2.c.right = vwidth;
+    r2.c.bottom = vheight;
+    if (stretched != 0){
+      r2.c.left = 0;
+      r2.c.top = 0;
+      r2.c.right = rwidth;
+      r2.c.bottom = rheight;
+    }
+
+    *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf);
+//printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
+    if (*mem_gid == 0){
+      if (reuse_mem){
+        WSGFbtronDeleteMemory((char*)buf2);
+      }else{
+        delete buf2;
+      }
+      return -1;
+    }
+    W ret = gcop_bmp(bgid,&r2,*mem_gid,&r,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+//  W ret = grst_bmp(*mem_gid,&r,&bmp,&r2,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+
+    if (reuse_mem){
+      WSGFbtronDeleteMemory((char*)buf2);
+    }else{
+      delete buf2;
+    }
+    gcls_env(bgid);
+  }
+  return 0;
+#if 0
   long stretched = 0;
   long vwidth = *w;
   long vheight = *h;
@@ -597,7 +863,7 @@ long WSGFbtronBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheigh
   *w = vwidth;
   *h = vheight;
 
-  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight);
+  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf);
 //printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
   if (*mem_gid == 0){
     return -1;
@@ -605,28 +871,8 @@ long WSGFbtronBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheigh
   long i,j;
   long ptr=0;
   unsigned char r,g,b;
-#if 0 //need not..
-  DEV_SPEC* dev_spec = WSGIbtronAppDev()->getDevSpec();
-  WSCulong red_shift = 0;
-  WSCulong red_bits = 0;
-  WSCulong green_shift = 0;
-  WSCulong green_bits = 0;
-  WSCulong blue_shift = 0;
-  WSCulong blue_bits = 0;
-  WSCbool speed_up = False;
-  if (dev_spec != NULL && ((dev_spec->attr & 0x3) == 1) &&
-      ((dev_spec->attr & 0x4) == 0)){
-    red_shift = (dev_spec->color[0] & 0xff00) >> 8;
-    green_shift = (dev_spec->color[1] & 0xff00) >> 8;
-    blue_shift = (dev_spec->color[2] & 0xff00) >> 8;
-    red_bits = (dev_spec->color[0] & 0xff);
-    green_bits = (dev_spec->color[1] & 0xff);
-    blue_bits = (dev_spec->color[2] & 0xff);
-    speed_up = True;
-  }
-printf("speed_up=%d dev_spec->attr=0x%x 0x%x 0x%x\n",speed_up,dev_spec->attr,dev_spec->attr & 0x3, dev_spec->attr & 0x4);
-#endif
   for(i=0; i<vheight; i++){
+    ptr = row_bytes*i;
     for(j=0; j<vwidth; j++){
       if (stretched != 0){
         ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -652,12 +898,182 @@ printf("speed_up=%d dev_spec->attr=0x%x 0x%x 0x%x\n",speed_up,dev_spec->attr,dev
       pt.y = i;
       gdra_pnt(*mem_gid,pt,pixval,G_STORE);
     }
-    ptr = row_bytes*i;
   }
   return 0;
+#endif
 }
-long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth,long rheight,long* w,long* h,long row_bytes){
+long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth,long rheight,long* w,long* h,long row_bytes,UB** _gid_buf1,UB** _gid_buf2,WSCbool reuse_mem){
+#if 1
+  long stretched = False;
+  long vwidth = *w;
+  long vheight = *h;
+  if (vwidth == 0 || vheight == 0){
+    vwidth = rwidth;
+    vheight = rheight;
+  }else{
+    if ((long)((double)rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)((double)vheight * rwidth/rheight);
+    }else{
+      vheight = (long)((double)vwidth * rheight/rwidth);
+    }
+    if (vwidth == rwidth && vheight == rheight){
+      stretched = False;
+    }else{
+      stretched = True;
+    }
+  }
+  *w = vwidth;
+  *h = vheight;
+  long i,j;
+  UB* buf2 = NULL;
+  UB* buf3 = NULL;
+  if (reuse_mem){
+    buf2 = (UB*)WSGFbtronAllocNewMemory( rheight * rwidth * 2);
+    buf3 = (UB*)WSGFbtronAllocNewMemory( rheight * rwidth * 2);
+  }else{
+    buf2 = new UB [rheight * rwidth * 2];
+    buf3 = new UB [rheight * rwidth * 2];
+  }
 
+  long cnt = 0;
+  long ptr = 0;
+  DEV_SPEC* dspec = WSGIbtronAppDev()->getDevSpec();
+  WSCbool duplicate = False;
+  if ((dspec->attr & 0xf) != 1){
+    duplicate = True;
+  }else{
+    if (dspec->color[0] == 0x0b05 &&
+        dspec->color[1] == 0x0506 &&
+        dspec->color[2] == 0x0005){
+      //OK
+    }else{
+      duplicate = True;
+    }
+  }
+
+  for(i=0; i<vheight; i++){
+    ptr = row_bytes*i;
+    for(j=0; j<vwidth; j++){
+      UB r = buf[ptr++];
+      UB g = buf[ptr++];
+      UB b = buf[ptr++];
+      UB a = buf[ptr++];
+      buf2[cnt] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
+      buf2[cnt+1] = (r & 0xf8) | ((g & 0xe0) >> 5);
+      if (a != 0){
+        a=0;
+        buf3[cnt++] = 0;
+        buf3[cnt++] = 0;
+      }else{
+        a = 0xff;
+        buf3[cnt++] = 0xff;
+        buf3[cnt++] = 0xff;
+      }
+//      buf3[cnt++] = ((a & 0x1c) << 3 )| (a & 0xf8) >> 3;
+//      buf3[cnt++] = (a & 0xf8) | ((a & 0xe0) >> 5);
+    }
+  }
+
+  BMP bmp;
+  bmp.planes = 1;
+  bmp.pixbits = 0x1010;
+  bmp.rowbytes = rwidth*2;
+  bmp.bounds.c.left = 0;
+  bmp.bounds.c.top = 0;
+  bmp.bounds.c.right = rwidth;
+  bmp.bounds.c.bottom = rheight;
+  bmp.baseaddr[0] = buf2;
+
+  BMP bmp2;
+  bmp2.planes = 1;
+  bmp2.pixbits = 0x1010;
+  bmp2.rowbytes = rwidth*2;
+  bmp2.bounds.c.left = 0;
+  bmp2.bounds.c.top = 0;
+  bmp2.bounds.c.right = rwidth;
+  bmp2.bounds.c.bottom = rheight;
+  bmp2.baseaddr[0] = buf3;
+
+
+  CSPEC   csp;
+  csp.attr = DA_COLOR_RGB;
+//  csp.info[0] = 0x1008;
+//  csp.info[1] = 0x0808;
+//  csp.info[2] = 0x0008;
+//  csp.info[3] = 0;
+  csp.info[0] = 0x0b05;
+  csp.info[1] = 0x0506;
+  csp.info[2] = 0x0005;
+  csp.info[3] = 0;
+  csp.colmap = NULL;
+
+  W bgid = gopn_mem(NULL,&bmp,(B*)(&csp));
+  W bgid2 = gopn_mem(NULL,&bmp2,(B*)(&csp));
+//printf("bgid=0x%x\n",bgid);
+  if (duplicate == False && stretched == False){
+    *mem_gid = bgid;
+    *_gid_buf1 = buf2;
+    *mem_gid2 = bgid2;
+    *_gid_buf2 = buf3;
+//printf(">>>> mem_gid=0x%x,0x%x  w,h=%d,%d\n",*mem_gid,*mem_gid2,vwidth,vheight);
+  }else{
+    RECT r;
+    RECT r2;
+    r.c.left = 0;
+    r.c.top = 0;
+    r.c.right = rwidth;
+    r.c.bottom = rheight;
+    r2.c.left = 0;
+    r2.c.top = 0;
+    r2.c.right = vwidth;
+    r2.c.bottom = vheight;
+    if (stretched != 0){
+      r2.c.left = 0;
+      r2.c.top = 0;
+      r2.c.right = rwidth;
+      r2.c.bottom = rheight;
+    }
+
+    *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf1);
+//printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
+    if (*mem_gid == 0){
+      if (reuse_mem){
+        WSGFbtronDeleteMemory((char*)buf2);
+        WSGFbtronDeleteMemory((char*)buf3);
+      }else{
+        delete buf2;
+        delete buf3;
+      }
+      return -1;
+    }
+    *mem_gid2 = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf2);
+    if (*mem_gid2 == 0){
+      gcls_env(*mem_gid);
+      if (reuse_mem){
+        WSGFbtronDeleteMemory((char*)buf2);
+        WSGFbtronDeleteMemory((char*)buf3);
+      }else{
+        delete buf2;
+        delete buf3;
+      }
+      return -1;
+    }
+    W ret = gcop_bmp(bgid,&r2,*mem_gid,&r,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+    ret = gcop_bmp(bgid2,&r2,*mem_gid2,&r,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+//  W ret = grst_bmp(*mem_gid,&r,&bmp,&r2,NULL,G_CVCOLOR | G_CVFORM | G_STORE);
+
+    if (reuse_mem){
+      WSGFbtronDeleteMemory((char*)buf2);
+      WSGFbtronDeleteMemory((char*)buf3);
+    }else{
+      delete buf2;
+      delete buf3;
+    }
+    gcls_env(bgid);
+    gcls_env(bgid2);
+  }
+  return 0;
+#else
   long stretched = 0;
   long vwidth = *w;
   long vheight = *h;
@@ -675,8 +1091,8 @@ long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
   *w = vwidth;
   *h = vheight;
 
-  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight);
-  *mem_gid2 = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight);
+  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf1);
+  *mem_gid2 = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight,_gid_buf2);
 //printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
   if (*mem_gid == 0){
     return -1;
@@ -685,6 +1101,7 @@ long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
   long ptr=0;
   unsigned char r,g,b,a;
   for(i=0; i<vheight; i++){
+    ptr = row_bytes*i;
     for(j=0; j<vwidth; j++){
       if (stretched != 0){
         ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -705,6 +1122,9 @@ long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
       pt.x = j;
       pt.y = i;
       gdra_pnt(*mem_gid,pt,pixval,G_STORE);
+      if (a != 0){
+        a=0xff;
+      }
       a = ~a;
       val = 0x10000000;
       val |= a << 16;
@@ -713,18 +1133,18 @@ long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
       gcnv_col(*mem_gid2,val,&pixval);
       gdra_pnt(*mem_gid2,pt,pixval,G_STORE);
     }
-    ptr = row_bytes*i;
   }
   return 0;
+#endif
 }
-long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,long* w,long* h){
+long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,W* pixmap2,long* w,long* h,UB** ptr,UB** ptr2){
 
   FILE* ifile = fopen(filename,"r");
   if (ifile == NULL){
     return -1;
   }
 
-  long reti = WSGFbtronBmpReadFpToPixmap(ifile, pixmap, w, h);
+  long reti = WSGFbtronBmpReadFpToPixmap(ifile, pixmap,pixmap2, w, h,ptr,ptr2);
   fclose(ifile);
   return reti;
 }
@@ -745,7 +1165,7 @@ void png_read_data_fn(png_structp png_ptr,png_bytep data,png_size_t length){
    if (check != length)
       png_error(png_ptr, "Read Error");
 }
-long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h){
+long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h,UB** ptr1,UB** ptr2){
 
     FILE* ifile = fopen(filename,"r");
     if (ifile == NULL){
@@ -939,12 +1359,12 @@ fprintf(stderr,"png read error: out of memory..\n");
     png_read_end(png_ptr,NULL);
     delete png_row_ptrs;
     if (components == 4){
-      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,png_width,png_height,w,h,png_row_bytes);
+      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,png_width,png_height,w,h,png_row_bytes,ptr1,ptr2,False);
     }else if (components == 3){
       *mem_gid2 = 0;
-      WSGFbtronBufToPixmap(buf, mem_gid,png_width,png_height,w,h,png_row_bytes);
+      WSGFbtronBufToPixmap(buf, mem_gid,png_width,png_height,w,h,png_row_bytes,ptr1,False);
     }else if (components == 2){
-      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,png_width,png_height,w,h,png_row_bytes);
+      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,png_width,png_height,w,h,png_row_bytes,ptr1,ptr2,False);
     }else{
       delete buf;
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -954,7 +1374,6 @@ fprintf(stderr,"png read error: out of memory..\n");
     delete buf;
     png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
     fclose(ifile);
-//printf("png mem_gid=0x%x w,h=%d,%d\n",mem_gid,*w,*h);
     return 0;
 }
 
@@ -1040,7 +1459,7 @@ void jpeg_stdio_src (j_decompress_ptr cinfo, FILE * infile){
 }
 #endif //BTRON_ADDED
 
-long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h){
+long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h,UB** ptr1,UB** ptr2,WSCbool reuse_mem){
     FILE* ifile = fopen(filename,"r");
     if (ifile == NULL){
       return -1;
@@ -1081,32 +1500,52 @@ long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long
 
     long components = jinf.output_components;
 
-    unsigned char* buf = new unsigned char[components * rheight * rwidth*sizeof(JSAMPLE)];
+    unsigned char* buf = NULL;
+    if (reuse_mem){
+      buf = (unsigned char*)
+         WSGFbtronAllocNewMemory(components * rheight * rwidth*sizeof(JSAMPLE));
+    }else{
+      buf = new unsigned char[components * rheight * rwidth*sizeof(JSAMPLE)];
+    }
+
     JSAMPROW rowptr[1];
+//printf("before read scanline..\n");
     long i;
     for(i=0; i< rheight; i++){
       rowptr[0] = (JSAMPROW)&(buf[i * rwidth * components]);
       jpeg_read_scanlines(&jinf, rowptr,1);
     }
+//printf("before read scanline done..\n");
     if (components == 4){
-      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE));
+      WSGFbtronBufToPixmap2(buf, mem_gid,mem_gid2,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE),ptr1,ptr2,reuse_mem);
     }else
     if (components == 3){
+//printf("before move scanline..\n");
       *mem_gid2 = 0;
-      WSGFbtronBufToPixmap(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE));
+      WSGFbtronBufToPixmap(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE),ptr1,reuse_mem);
     }else
     if (components == 1){
       *mem_gid2 = 0;
-      WSGFbtronBufToPixmapG(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE));
+      WSGFbtronBufToPixmapG(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE),ptr1,reuse_mem);
 
     }else{
-      delete buf;
+      if (reuse_mem){
+        WSGFbtronDeleteMemory((char*)buf);
+      }else{
+        delete buf;
+      }
       jpeg_finish_decompress(&jinf);
       jpeg_destroy_decompress(&jinf);
       fclose(ifile);
       return -1;
     }
-    delete buf;
+//printf("before move scanline done..\n");
+    if (reuse_mem){
+      WSGFbtronDeleteMemory((char*)buf);
+    }else{
+      delete buf;
+    }
+
     jpeg_finish_decompress(&jinf);
     jpeg_destroy_decompress(&jinf);
     fclose(ifile);
