@@ -579,14 +579,16 @@ void WSDbtronScrFrameDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,
 #ifndef F_MEMGID
   WSDbtronFormDev::_setup_pixmap(x,y,w,h,bg,img_no);
 #else
-  WSDbtronFormDev::_setup_pixmap(x-_h_scr_pos,y-_v_scr_pos,w,h,bg,img_no);
+//  WSDbtronFormDev::_setup_pixmap(x-_h_scr_pos,y-_v_scr_pos,w,h,bg,img_no);
+  WSDbtronFormDev::_setup_pixmap(x,y,w,h,bg,img_no);
 #endif
 }
 void WSDbtronScrFrameDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
 #ifndef F_MEMGID
   WSDbtronFormDev::_clear_area(x,y,w,h);
 #else
-  WSDbtronFormDev::_clear_area(x - _h_scr_pos,y - _v_scr_pos,w,h);
+//  WSDbtronFormDev::_clear_area(x - _h_scr_pos,y - _v_scr_pos,w,h);
+  WSDbtronFormDev::_clear_area(x,y,w,h);
 #endif
 } 
 
