@@ -157,10 +157,26 @@ dbprintf("WSDbtronWinDev::setValue need execute expose..%s:%d\n",__FILE__,__LINE
                }
              }
              break;
-    case WSDEV_BACKCOLOR:
+    case WSDEV_BACKCOLOR:{
              _bg = *(short*)data;
 dbprintf("WSDbtronWinDev::setValue back color..%s:%d bg=%d\n",__FILE__,__LINE__,_bg);
+             if (_window != NULL){
+               W bgc = 0x10000000;
+               WSDcolor* color = WSGIappColorSet()->getColor(_bg);
+               if (color != NULL){
+                 bgc = color->getValue1();
+               }
+               PAT pat;
+               pat.spat.kind = 0;
+               pat.spat.hsize = 16;
+               pat.spat.vsize = 16;
+               pat.spat.fgcol = bgc;
+               pat.spat.bgcol = bgc;
+               pat.spat.mask = FILL100;
+               wset_bgp(_window,&pat);
+             }
              break;
+    }
     case WSDEV_BACK_PIXMAP:
              _bg_pix = *(short*)data;
              if (getVisible() != False){
@@ -317,7 +333,20 @@ dbprintf("### WSDbtronWinDev::createWindow %s vis=%d attr=0x%x\n",getAttachedCli
     tstr2 = WSGFbtronStringToTc2( _title_str.getString(WS_EN_UTF8),&tclen);
 dbprintf("WSDbtronWinDev::createWindow title=#%s# tclen=%d\n",_title_str.getString(),tclen);
   }
-
+  W bgc = 0x10000000;
+  WSDcolor* color = WSGIappColorSet()->getColor(_bg);
+  if (color != NULL){
+    bgc = color->getValue1();
+  }
+  PAT pat;
+  pat.spat.kind = 0;
+  pat.spat.hsize = 16;
+  pat.spat.vsize = 16;
+  pat.spat.fgcol = bgc;
+  pat.spat.bgcol = bgc;
+  pat.spat.mask = FILL100;
+//  wset_bgp(_window,&pat);
+ 
   if (_top_float != False && _no_frame != False){
 //    UW attr = WA_SUBW | WA_NOFRAME | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
 //    UW attr = WA_NOFRAME | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
@@ -333,30 +362,35 @@ dbprintf("WSDbtronWinDev::createWindow title=#%s# tclen=%d\n",_title_str.getStri
     if (pid == 0){
 //dbprintf("WSDbtronWinDev::createWindow begin wopn_wnd\n");
       UW attr = WA_NOFRAME | WA_SUBW | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
-      _window = wopn_wnd(attr,parent,&rect,NULL,2,NULL,NULL,NULL);
+      _window = wopn_wnd(attr,parent,&rect,NULL,2,NULL,&pat,NULL);
     }else{
 //dbprintf("WSDbtronWinDev::createWindow begin wopn_wnd2\n");
       UW attr = WA_NOFRAME | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
-      _window = wopn_wnd(attr,0,&rect,NULL,2,NULL,NULL,NULL);
+      _window = wopn_wnd(attr,0,&rect,NULL,2,NULL,&pat,NULL);
     }
 //    _window = wopn_pwd(&rect);
 //printf("wopn_pwd!!! _window=0x%x parent=0x%x\n",_window,parent);
   }else{
     if (tstr2 != NULL){
 //dbprintf("WSDbtronWinDev::createWindow begin wopn_wnd3\n");
-      _window = wopn_wnd(attr,0,&rect,NULL,2,tstr2,NULL,NULL);
-      delete tstr2;
+      _window = wopn_wnd(attr,0,&rect,NULL,2,tstr2,&pat,NULL);
     }else{
 //dbprintf("WSDbtronWinDev::createWindow begin wopn_wnd4\n");
-      _window = wopn_wnd(attr,0,&rect,NULL,2,NULL,NULL,NULL);
+      _window = wopn_wnd(attr,0,&rect,NULL,2,NULL,&pat,NULL);
     }
 //printf("wopn_wnd!!! _window=0x%x\n",_window);
   }
+  if (tstr2 != NULL){
+    delete tstr2;
+    tstr2 = 0;
+  }
 dbprintf("WSDbtronWinDev::createWindow _window=0x%x\n",_window);
 //printf("WSDbtronWinDev::createWindow %d,%d,%d,%d w=0x%x\n",_w_x,_w_y,_w_w,_w_h,_window);
 //dbprintf("WSDbtronWinDev::createWindow %d,%d,%d,%d\n",_w_x,_w_y,_w_w,_w_h);
   if (_window > 0){
+//printf("createWindow win=0x%x\n",_window);
 //    WSGIbtronAppDev()->addWindow(_window);
+    wset_bgp(_window,&pat);
     RECT rect;
     W rv = wget_wrk(_window,&rect);
     if (rv == E_OK){
@@ -376,7 +410,6 @@ dbprintf("WSDbtronWinDev::createWindow _window=0x%x\n",_window);
 
     setPixmapStyle(WS_DYNAMIC_PIXMAP);
     createPixmap();
-
     _expose_list.add((void*)_window);
   }
   return WS_NO_ERR;
@@ -500,7 +533,7 @@ dbprintf("WSDbtronFormDev::createPixmap() gcls_env=0x%x\n",ret);
     h = 1;
   }
                                                                                 
-  _mem_gid = WSGIbtronAppDev()->getNewMemGid(w,h);
+  _mem_gid = WSGIbtronAppDev()->getNewMemGid(w,h,&_mem_gid_buf);
   WSDwindowDev::createPixmap();
 //dbprintf("WSDbtronWinDev::createPixmap() wh=%d,%d %s\n",w,h,getAttachedClient()->getInstanceName());
   _setup_pixmap(0,0,w,h,_bg,_bg_pix);
@@ -520,6 +553,8 @@ dbprintf("WSDbtronFormDev::createPixmap() gcls_env=0x%x\n",ret);
 }
 long WSDbtronWinDev::destroyPixmap(){
   if (_mem_gid != NULL){
+    delete _mem_gid_buf;
+    _mem_gid_buf = NULL;
     ERR ret = gcls_env(_mem_gid);
 dbprintf("WSDbtronFormDev::destroyPixmap() gcls_env=0x%x\n",ret);
     _mem_gid = NULL;
