@@ -551,6 +551,50 @@ int WSGFwsInitialize(){
   return 0;
 }
 
+static long mem_initialized = False;
+static char* buf[16];
+static long buf_size[16];
+static long buf_use[16];
 
+char* WSGFbtronAllocNewMemory(long size){
 
+  if (mem_initialized == False){
+    int i;
+    for(i=0; i<16; i++){
+      buf[i] = NULL;
+      buf_size[i] = 0;
+      buf_use[i] = 0;
+    }
+    mem_initialized = True;
+  }
+  int i;
+  for(i=0; i<16; i++){
+    if (buf[i] != NULL && buf_size[i] > size-1 && buf_use[i] == False){
+      buf_use[i] = True;
+      return buf[i];
+    }
+  }
+  for(i=0; i<16; i++){
+    if (buf[i] == NULL){
+      buf[i] = new char[size];
+      buf_size[i] = size;
+      buf_use[i] = True;
+      return buf[i];
+    }
+  }
+  buf[0] = new char[size];
+  buf_size[0] = size;
+  buf_use[0] = True;
+  return buf[i];
+}
+void WSGFbtronDeleteMemory(char* ptr){
 
+  int i;
+  for(i=0; i<16; i++){
+    if (buf[i] == ptr){
+      buf_use[i] = False;
+      return;
+    }
+  }
+  delete ptr;
+}
