@@ -382,7 +382,7 @@ int  ScrnX::line_rgb(Vector2D p1, Vector2D p2, int r, int g, int b)
 int  ScrnX::point(Vector2D p, int color)
 {
 #ifdef WS
-printf("ScrnX::point...%d\n",color);
+//printf("ScrnX::point...%d\n",color);
 #else //WS
     XPoint xp;
 
@@ -447,7 +447,7 @@ int ScrnX::polygon(Vector2D *vp, int vn, int color)
 int ScrnX::polygon_rgb(Vector2D *vp, int vn, int r, int g, int b)
 {
 #ifdef WS
-printf("ScrnX::polycon_rgb %d %d %d\n",r,g,b);
+//printf("ScrnX::polycon_rgb %d %d %d\n",r,g,b);
 #else //WS
     XPoint xp[24];
     unsigned long pixel;
