@@ -612,7 +612,7 @@ void WSDnxFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCushort
   if (getParentMemContext() == 0){
     return;
   }
-  _setup_pixmap(x,x,w,h,_bg,_bg_pix);
+  _setup_pixmap(x,y,w,h,_bg,_bg_pix);
   exposeExecute(x,y,w,h);
   short px,py;
   getAbstPos(&px,&py);
