@@ -19,8 +19,8 @@
 #define FALSE 0
 #define BOOL  int
 
-//static int xu_locale_encoding = XU_CONV_ISO8859(1);
-static int xu_locale_encoding = XU_CONV_EUCJP;
+static int xu_locale_encoding = XU_CONV_ISO8859(1);
+//static int xu_locale_encoding = XU_CONV_EUCJP;
 int XUGetLocale(){
   return xu_locale_encoding;
 }
