@@ -500,7 +500,8 @@ long WSCsheet::initialize()
 	_edit->initialize();
 	_edit->setPropertyV(WSNshadowType, (char)WS_SHADOW_BORDER);
 	_edit->setPropertyV(WSNshadowThickness, (WSCuchar)0);
-
+	_edit->setPropertyV(WSNmaxLength, 32767);
+	
 	WSCprocedure* op3 = new WSCprocedure("focus-ch", WSEV_FOCUS_CH);
 	op3->setInternal(True);
 	op3->setFunction(onEditFocusChange, "onEditFocusChange");
@@ -872,7 +873,7 @@ void WSCsheet::onSheetExpose(WSCbase* obj)
 	for(row = ss->getTopVisibleRow(); (row < ss->getRowCount()) && (rect.y < height); row++) {
 		rect.x = x;
 		rect.height = ss->getRowHeight(row);
-		for(col = ss->_topVisibleCol; (col < ss->getColumnCount()) && (rect.x < width); col++) {
+		for(col = ss->getTopVisibleCol(); (col < ss->getColumnCount()) && (rect.x < width); col++) {
 			rect.width = ss->getColumnWidth(col);
 			ss->drawCell(dev, col, row, rect);
 			rect.x += rect.width;
@@ -1638,7 +1639,9 @@ long WSCsheet::setFocusCell(long col, long row)
 	oldRow = _focusRow;
 	_focusCol = col;
 	_focusRow = row;
-
+	setCol(col);
+	setRow(row);
+	
 	focusCellVisible(oldCol, oldRow, col, row);
 
 	draw();
@@ -2175,6 +2178,8 @@ long WSCsheet::setTopVisibleRow(long row)
 	}
 	_topVisibleRow = row;
 
+	adjustScrollBar();
+
 	draw();
 
 	return WS_NO_ERR;
@@ -2186,6 +2191,8 @@ long WSCsheet::setTopVisibleCol(long col)
 	}
 	_topVisibleCol = col;
 
+	adjustScrollBar();
+
 	draw();
 	return WS_NO_ERR;
 }
@@ -2221,7 +2228,9 @@ long WSCsheet::setCellText(const WSCstring& text)
 		}
 		cell->text = text;
 	}
-
+	if(_edit && getEditEnabled()) {
+		focusCellVisible(-1, -1, getCol(), getRow());
+	}
 	draw();
 	
 	return WS_NO_ERR;
@@ -2242,6 +2251,9 @@ long WSCsheet::setCellText(long col, long row, const WSCstring& text)
 	}
 	cell->text = text;
 
+	if(_edit && getEditEnabled()) {
+		focusCellVisible(-1, -1, getCol(), getRow());
+	}
 	draw();
 	
 	return WS_NO_ERR;
@@ -2309,7 +2321,9 @@ void WSCsheet::adjustScrollBar()
 	_hscrBar->setPropertyV(WSNmaximum,(WSCulong)(_cols + val - getFixedColumnCount() - 1));
 	_hscrBar->setPropertyV(WSNsliderSize,(WSCulong)val);
 	_hscrBar->setPropertyV(WSNincrement, (WSCushort)1);
-
+	_hscrBar->setPropertyV(WSNvalue, (WSCulong)getTopVisibleCol()-1);
+	
+	
 	_vscrBar->setProperty(WSNx, _clientWidth + _shadow_thick);
 	_vscrBar->setProperty(WSNy, _shadow_thick);
 	_vscrBar->setProperty(WSNwidth, _bar_thick);
@@ -2318,6 +2332,7 @@ void WSCsheet::adjustScrollBar()
 	_vscrBar->setPropertyV(WSNmaximum,(WSCulong)(_rows + val - getFixedRowCount() - 1));
 	_vscrBar->setPropertyV(WSNsliderSize,(WSCulong)val);
 	_vscrBar->setPropertyV(WSNincrement, (WSCushort)1);
+	_vscrBar->setPropertyV(WSNvalue, (WSCulong)getTopVisibleRow()-1);
 
 
 }
@@ -2596,6 +2611,7 @@ void WSCsheet::focusCellVisible(long oldCol, long oldRow, long newCol, long newR
 				_hscrBar->getPropertyV(WSNvalue, &val);
 				if((long)val < getColumnCount() - getFixedColumnCount()) {
 					_hscrBar->setPropertyV(WSNvalue, ++val);
+					_topVisibleCol = val;
 					getFocusCellRect(&rect);
 				}
 			}
@@ -2603,6 +2619,7 @@ void WSCsheet::focusCellVisible(long oldCol, long oldRow, long newCol, long newR
 				_vscrBar->getPropertyV(WSNvalue, &val);
 				if((long)val < getRowCount() - getFixedRowCount()) {
 					_vscrBar->setPropertyV(WSNvalue, ++val);
+					_topVisibleRow = val;
 					getFocusCellRect(&rect);
 				}
 			}
@@ -2620,15 +2637,15 @@ void WSCsheet::focusCellVisible(long oldCol, long oldRow, long newCol, long newR
 			_edit->setPropertyV(WSNfont, (unsigned char)cell->fontno);
 
 			_edit->setFocus(False);
-//			_edit->setVisible((getSelectCell() ? False : True));
 
 			if(fval) {
-				_edit->setSelect(0, (short)strlen(text.getString()));
-				_edit->setPropertyV(WSNcursorPos, (short)strlen(text.getString()));
+				_edit->setPropertyV(WSNcursorPos, (short)text.getChars());
 			}
-
 			_edit->setVisible((getSelectCell() ? False : True));
-			_edit->setFocus(True);
+
+			if(_edit->getVisible()) {
+				_edit->setFocus(True);
+			}
 		}
 	} else {
 		_edit->setVisible(False);
@@ -2889,7 +2906,7 @@ long WSCsheet::setDbStmtString(WSCstring& stmt)
 }
 long WSCsheet::dbRefresh()
 {
-	return dbDataSet();;
+	return dbDataSet();
 }
 ///////////////////////////////////////////////////////////////////////////////
 // database local function
