@@ -80,7 +80,7 @@ WSDwinKeyboard::~WSDwinKeyboard(){
 
 long WSDwinKeyboard::initialize(){
   _himc = ImmCreateContext();
-  setSelectedString("");
+  WSDkeyboard::setSelectedString("");
   return WS_NO_ERR;
 }
 
