@@ -315,8 +315,12 @@ void WSDwinformDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,long
       BitBlt(_dc,dx,dy,w,h,_mdc,x,y,SRCCOPY);
     } 
   }else{ 
-//WSMFtrace("WSDwinformDev::_copy_area %d,%d,%d,%d ->%d,%d XXZZp3\n",x,y,w,h,dx,dy);
+//WSMFtrace("WSDwinformDev::_copy_area %d,%d,%d,%d ->%d,%d XXZZp3 _dc=0x%x _mdc=0x%x \n",x,y,w,h,dx,dy,_dc,_mdc);
     BitBlt(_dc,dx,dy,w,h,_dc,x,y,SRCCOPY);
+    if (_mdc != 0){
+      SelectClipRgn(_mdc,(HRGN)0);
+      BOOL ret = BitBlt(_mdc,dx,dy,w,h,_mdc,x,y,SRCCOPY);
+    }
 #if 0
     HDC mdc2 = CreateCompatibleDC(_dc);
     HBITMAP pix = CreateCompatibleBitmap(_dc,w,h);
@@ -420,6 +424,8 @@ void WSDwinformDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
      palette_bk = SelectPalette(dc,palette,False);
      RealizePalette(dc);
   }
+
+
   if (_bg_image != 0){
     WSDimage* img = WSGIappImageSet()->getImage(_back_pixno);
 //printf("WSDwinformDev::_clear_area XXZZ11 _back_pixno=%d 0x%x\n",_back_pixno,img);
@@ -439,8 +445,22 @@ void WSDwinformDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
       if (ph < 1){
         ph = 1;
       }
+
       HRGN rgn = CreateRectRgn(x,y,x+w,y+h);
       SelectClipRgn(dc,rgn);
+
+      HBITMAP bg_mask = (HBITMAP)img->getValue3();
+      HDC mdc3 = 0;
+      HBITMAP bbk2 = 0;
+      if (bg_mask != (HBITMAP)-1 && bg_mask != (HBITMAP)0){
+        mdc3 = CreateCompatibleDC(dc);
+        bbk2 = (HBITMAP)SelectObject(mdc3,(void*)bg_mask);
+        SetMapMode(mdc3,GetMapMode(dc));
+        HBRUSH brush = CreateSolidBrush(_bg_cref);
+        FillRect(dc,&r,brush);
+        DeleteObject(brush);
+      }
+
 //printf("WSDwinformDev::_clear_area XXZZ _bg_image=0x%x %d,%d\n",_bg_image,pw,ph);
 //WSMFtrace("WSDwinformDev::_clear_area XXZZp1  x,y,w,h=%d,%d,%d,%d  pw,ph=%d,%d\n",x,y,w,h,pw,ph);
       long dx,dy;
@@ -448,7 +468,12 @@ void WSDwinformDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
         for(dy = 0; dy < (signed long)(y+h); dy = dy + ph){
           if ( dx < (signed long)(x+w) && dy < (signed long)(y+h)){
 //WSMFtrace("WSDwinformDev::_clear_area XXZZp1c x,y,w,h=%d,%d,%d,%d \n",dx,dy,pw,ph,pw,ph);
-            BitBlt(dc,dx,dy,pw,ph,mdc2,0,0,SRCCOPY);
+            if (mdc3 == 0){
+              BitBlt(dc,dx,dy,pw,ph,mdc2,0,0,SRCCOPY);
+            }else{
+extern void _draw_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP pix2,HPALETTE pl,int x,int y,int w,int h);
+              _draw_image_with_mask(dc,mdc2,mdc3,_bg_image, bg_mask,palette,dx,dy,pw,ph);
+            }
           }
         }
       }
@@ -457,6 +482,10 @@ void WSDwinformDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
       SelectObject(mdc2,(void*)bbk);
       DeleteObject((void*)rgn);
       DeleteDC(mdc2);
+      if (mdc3 != 0){
+        SelectObject(mdc3,(void*)bbk2);
+        DeleteDC(mdc3);
+      }
     }
   }else{
     HBRUSH brush = CreateSolidBrush(_bg_cref);
@@ -523,6 +552,7 @@ void WSDwinformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short
   r.top  =y;
   r.right = x+w;
   r.bottom = y+h;
+
   if (_bg_image != (HBITMAP)0){
     WSDimage* img = WSGIappImageSet()->getImage(_back_pixno);
     if (img != NULL){
@@ -545,12 +575,31 @@ void WSDwinformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short
       }
       HRGN rgn = CreateRectRgn(x,y,x+w,y+h);
       SelectClipRgn(_mdc,rgn);
+
+      HBITMAP bg_mask = (HBITMAP)img->getValue3();
+      HDC mdc3 = 0;
+      HBITMAP bbk2 = 0;
+      if (bg_mask != (HBITMAP)-1 && bg_mask != (HBITMAP)0){
+        mdc3 = CreateCompatibleDC(_mdc);
+        bbk2 = (HBITMAP)SelectObject(mdc3,(void*)bg_mask);
+        SetMapMode(mdc3,GetMapMode(_mdc));
+
+        HBRUSH br = CreateSolidBrush(bgref);
+        FillRect(_mdc,&r,br);
+        DeleteObject((void*)br);
+      }
+
 //WSMFtrace("WSDwinformDev::_setup_pixmap XXZZp1  x,y,w,h=%d,%d,%d,%d\n",x,y,w,h);
       long dx,dy;
       for(dx = 0; dx < x+w; dx = dx + pw){
         for(dy = 0; dy < y+h; dy = dy + ph){
           if ( dx < x+w && dy < y+h){
-            BitBlt(_mdc,dx,dy,pw,ph,mdc2,0,0,SRCCOPY);
+            if (mdc3 == 0){
+              BitBlt(_mdc,dx,dy,pw,ph,mdc2,0,0,SRCCOPY);
+            }else{
+extern void _draw_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP pix2,HPALETTE pl,int x,int y,int w,int h);
+              _draw_image_with_mask(_mdc,mdc2,mdc3,_bg_image, bg_mask,pl,dx,dy,pw,ph);
+            }
           }
         }
       }
@@ -558,14 +607,16 @@ void WSDwinformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short
       SelectObject(mdc2,(void*)bbk);
       DeleteObject((void*)rgn);
       DeleteDC(mdc2);
+      if (mdc3 != 0){
+        SelectObject(mdc3,(void*)bbk2);
+        DeleteDC(mdc3);
+      }
     }
   }else{
-//printf("WSDwinformDev::_setup_pixmap XXZZredraw1 filled this=0x%x w,h=%d,%d\n",this,w,h);
     HBRUSH br = CreateSolidBrush(bgref);
     SelectClipRgn(_mdc,(HRGN)0);
     FillRect(_mdc,&r,br);
     DeleteObject((void*)br);
-//WSMFtrace("WSDwinformDev::_setup_pix  XXZZp3 %d %d %d %d _mdc=0x%x\n",x,y,w,h,_mdc);
   }
   if (plbk != (HPALETTE)0){
     SelectPalette(_mdc,plbk,FALSE);
