@@ -102,7 +102,9 @@ void timerop(WSCbase* object){
   }
 
 //update 줿ΤߡIPΥ󥹥󥹤ΤߤUDP 롣
-  if (WSGIappObjectList()->getRemoteInstanceListUpdated() != False){
+static long cnt = 0;
+  cnt++;
+  if (WSGIappObjectList()->getRemoteInstanceListUpdated() != False || cnt % 10 == 0){
     WSGIappObjectList()->setRemoteInstanceListUpdated(False);
     WSClistData* rlist = WSGIappObjectList()->getRemoteInstanceList();
     long num = rlist->getNum();
@@ -131,10 +133,27 @@ void timerop(WSCbase* object){
       strncpy((char*)&buffer[144],item->getPublicInstanceName(),64);
       buffer[207] = 0;
 //printf("send--------!!!! %x %x\n",buffer[1],buffer[0]);
-//printf("send------------!!!! %d %s %s\n",port,(char*)&buffer[6],(char*)item->getInstanceName());
       long ret = newvudp_000->write(buffer,256);
+//printf("send------------!!!! %d %s %s send_len=%d\n",port,(char*)&buffer[6],(char*)item->getInstanceName(),ret);
       if (ret != 256){
-        break;
+        long total_len = ret;
+        long cnt = 0;
+        while(1){
+          long ret = newvudp_000->write(&buffer[total_len],256-total_len);
+//printf("send2-----------!!!! %d %s %s send_len=%d\n",port,(char*)&buffer[6],(char*)item->getInstanceName(),ret);
+          if (ret < 0){
+            return;
+          }
+          if (total_len + ret != 256){
+            cnt++;
+            total_len += ret; 
+          }else{
+            break;
+          }
+          if (cnt > 10){
+            return;
+          }
+        }
       }
     }
   }
