@@ -458,6 +458,28 @@ long WSDxappDev::dispatchEvent(){
 long WSDxappDev::processEvent(){
   return _dispatchEvent(False);
 }
+WSCuchar hex_to_char(char* buffer){
+  WSCuchar val = 0;
+  if ( '0' <= buffer[0] && buffer[0] <='9'){
+    val |= ((buffer[0] - '0') << 4);
+  }
+  if ( 'a' <= buffer[0] && buffer[0] <='f'){
+    val |= ((buffer[0] - 'a' + 10) << 4);
+  }
+  if ( 'A' <= buffer[0] && buffer[0] <='F'){
+    val |= ((buffer[0] - 'A' + 10) << 4);
+  }
+  if ( '0' <= buffer[1] && buffer[1] <='9'){
+    val |= (buffer[1] - '0');
+  }
+  if ( 'a' <= buffer[1] && buffer[1] <='f'){
+    val |= (buffer[1] - 'a' + 10);
+  }
+  if ( 'A' <= buffer[1] && buffer[1] <='F'){
+    val |= (buffer[1] - 'A' + 10);
+  }
+  return val;
+}
 long WSDxappDev::_dispatchEvent(WSCbool wait,XEvent* ev){
 dbprintf("WSDxappDev::_dispatchEvent %s:%d start\n",__FILE__,__LINE__);
 
@@ -556,6 +578,8 @@ static Atom prop = 0;
 static Atom prop2 = 0;
 static Atom prop3 = 0;
 static Atom prop4 = 0;
+static Atom prop5 = 0;
+static Atom prop6 = 0;
     if (prop == 0){
       prop = XInternAtom(_disp,"TEXT",False);
     }
@@ -568,8 +592,17 @@ static Atom prop4 = 0;
     if (prop4 == 0){
       prop4 = XInternAtom(_disp,"TARGETS",False);
     }
+    if (prop5 == 0){
+      prop5 = XInternAtom(_disp,"UTF8_STRING",False);
+    }
+    if (prop6 == 0){
+      prop6 = XInternAtom(_disp,"text/plain;charset=UTF-8",False);
+//      prop6 = XInternAtom(_disp,"text/plain;charset=ISO-10646-UCS-2",False);
+    }
 
-
+//  if (e.type == PropertyNotify){
+//printf("PropertyChange atom=0x%x\n",e.xproperty.atom);
+//  }
   if (e.type == SelectionRequest){
 //printf("xappdev:: request data..\n");
     XSelectionEvent se;
@@ -579,15 +612,22 @@ static Atom prop4 = 0;
     se.target = e.xselectionrequest.target;
     se.time = e.xselectionrequest.time;
     se.property = e.xselectionrequest.property;
-
+//printf("SelectionReg target=0x%x prop=0x%x prop2=0x%x prop3=0x%x prop4=0x%x prop5=0x%x\n", e.xselectionrequest.target,prop,prop2,prop3,prop4,prop5);
     if (e.xselectionrequest.selection == XA_PRIMARY &&
         e.xselectionrequest.target == prop4 ){
-       Atom ar[3];
+       Atom ar[4];
+//TEST
+       ar[0] = prop5;
+       ar[1] = prop2;
+       ar[2] = prop;
+       ar[3] = prop3;
+#if 0
        ar[0] = prop4;
        ar[1] = prop2;
        ar[2] = prop3;
+#endif
       XChangeProperty(_disp,se.requestor,se.property,se.target,
-                      32,PropModeReplace,(unsigned char*)ar,3);
+                      32,PropModeReplace,(unsigned char*)ar,4);
 //TEST 4 -> 3
 
     }else if (e.xselectionrequest.selection == XA_PRIMARY &&
@@ -598,12 +638,73 @@ static Atom prop4 = 0;
                       8,PropModeReplace,(unsigned char*)str,strlen(str)+1);
 //printf("xappdev:: send data..#%s# %d %d %d\n",str,XA_PRIMARY,XA_STRING,se.property);
     }else if (e.xselectionrequest.selection == XA_PRIMARY &&
-        e.xselectionrequest.target == XA_STRING ){
+              e.xselectionrequest.target == XA_STRING ){
+//printf("send.. string target=0x%x XA_STRING\n",e.xselectionrequest.target);
       char* str = WSGIappKeyboard()->WSDkeyboard::getSelectedString(
-         WSGIappLocaleSet()->getSystemLocaleEncoding());
+                               WSGIappLocaleSet()->getSystemLocaleEncoding());
+      WSCushort* ustr = WSGFgetUCS2(str,
+                               WSGIappLocaleSet()->getSystemLocaleEncoding());
+      int i;
+      long ulen = WSGFstrlenUCS2(ustr);
+      WSCbool need_convert = False;
+      char* str2 = new char[ulen*8+1];
+      int cnt = 0;
+      for(i=0; i<ulen; i++){
+        if (ustr[i] & 0xff00){
+          need_convert = True;
+          str2[cnt++] = '\\';
+          str2[cnt++] = 'x';
+          str2[cnt++] = '{';
+          WSCuchar val = ((ustr[i] >> 12) & 0xf);
+          if (val >= 10){
+            val += ('A' -10);
+          }else{
+            val += '0';
+          }
+          str2[cnt++] = val;
+          val = ((ustr[i] >> 8) & 0xf);
+          if (val >= 10){
+            val += ('A' -10);
+          }else{
+            val += '0';
+          }
+          str2[cnt++] = val;
+		  val = ((ustr[i] >> 4) & 0xf);
+		  if (val >= 10){
+            val += ('A' -10);
+          }else{
+            val += '0';
+          }
+          str2[cnt++] = val;
+          val = (ustr[i] & 0xf);
+          if (val >= 10){
+            val += ('A' -10);
+          }else{
+            val += '0';
+          }
+          str2[cnt++] = val;
+          str2[cnt++] = '}';
+        }else{
+          str2[cnt] = ustr[i];
+          cnt++;
+        }
+        str2[cnt] = 0;
+      }
+      if (need_convert == False){
+        XChangeProperty(_disp,se.requestor,se.property,se.target,
+                      8,PropModeReplace,(unsigned char*)str,strlen(str)+1);
+      }else{
+        XChangeProperty(_disp,se.requestor,se.property,se.target,
+                      8,PropModeReplace,(unsigned char*)str2,strlen(str2)+1);
+      }
+      delete str2;
+      delete ustr;
+    }else if (e.xselectionrequest.selection == XA_PRIMARY &&
+         e.xselectionrequest.target == prop5 ){
+//printf("send.. string target=0x%x UTF8_STRING\n",e.xselectionrequest.target);
+      char* str = WSGIappKeyboard()->WSDkeyboard::getSelectedString(WS_EN_UTF8);
       XChangeProperty(_disp,se.requestor,se.property,se.target,
                       8,PropModeReplace,(unsigned char*)str,strlen(str)+1);
-
 //printf("xappdev:: send data2..#%s# %d %d %d\n",str,XA_PRIMARY,XA_STRING,se.property);
 #if 0
     }else if (e.xselectionrequest.selection == XA_PRIMARY &&
@@ -626,6 +727,7 @@ static Atom prop4 = 0;
 dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
     return WS_NO_ERR;
   }
+//printf("e.type=0x%x\n",e.type);
   if (e.type == SelectionNotify){
 
 
@@ -634,29 +736,88 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
     int fmt;
     WSCulong nitems;
     WSCulong left;
+//printf("selection=0x%x target=0x%x prop=0x%x\n",e.xselection.selection,e.xselection.target,e.xselection.property);
+//printf("prop=0x%x prop2=0x%x prop3=0x%x prop4=0x%x prop5=0x%x prop6=0x%x PRI=0x%x ST=0x%x\n",prop,prop2,prop3,prop4,prop5,prop6,XA_PRIMARY,XA_STRING);
 #if 0
-//    if (e.xselectionrequest.selection == prop3){
-    if (e.xselectionrequest.selection == prop4){
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == XA_STRING &&
+        e.xselection.property == prop2){
+      XConvertSelection(_disp,XA_PRIMARY,prop4,XA_STRING,e.xselection.requestor,e.xselection.time);
+    }else
+#endif
+
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == prop4){
+//printf("convert!!! target!!!\n");
       Atom *ar;
-      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop2,
-                              0,8192,False,XA_ATOM,&type,&fmt,&nitems,
+      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop4,
+                              0,8192,False,AnyPropertyType,&type,&fmt,&nitems,
                               &left,(WSCuchar**)&ar) == Success &&
           nitems > 0){
-        int i; 
+//printf("convert!!! target!!! OK!!\n");
+//printf("prop2=0x%x prop2=%s\n",prop2,XGetAtomName(_disp,prop2));
+        int i;
+        WSCbool fl = False;
         for(i=0;i<nitems;i++){
-          if (ar[i] == prop2 || ar[i] == prop3){
-            XConvertSelection(_disp,XA_PRIMARY,ar[i],prop,XtWindow(appWidget()),
-                e.xselection.time);
+//printf("ar[%d]=0x%x %s =0x%x\n",i,ar[i],XGetAtomName(_disp,ar[i]),prop6);
+          if (ar[i] == prop6){
+//printf("ar[%d]=0x%x %s hit!\n",i,ar[i],XGetAtomName(_disp,ar[i]));
+            fl = True;
+            XConvertSelection(_disp,XA_PRIMARY,prop6,prop6,e.xselection.requestor,e.xselection.time);
             break;
           }
         }
+        if (fl == False){
+          for(i=0;i<nitems;i++){
+//printf("ar[%d]=0x%x %s =0x%x\n",i,ar[i],XGetAtomName(_disp,ar[i]),prop5);
+            if (ar[i] == prop5){
+//printf("ar[%d]=0x%x %s hit\n",i,ar[i],XGetAtomName(_disp,ar[i]));
+              fl = True;
+              XConvertSelection(_disp,XA_PRIMARY,prop2,prop2,e.xselection.requestor,e.xselection.time);
+              break;
+            }
+          }
+        }
+        if (fl == False){
+          for(i=0;i<nitems;i++){
+//printf("ar[%d]=0x%x %s =0x%x\n",i,ar[i],XGetAtomName(_disp,ar[i]),prop2);
+            if (ar[i] == prop){
+//printf("ar[%d]=0x%x %s hit\n",i,ar[i],XGetAtomName(_disp,ar[i]));
+              fl = True;
+              XConvertSelection(_disp,XA_PRIMARY,prop,prop,e.xselection.requestor,e.xselection.time);
+              break;
+            }
+          }
+        }
       }
     }else
-#endif
-    if (e.xselectionrequest.selection == XA_STRING){
 
-      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop2,
-                              0,8192,False,XA_STRING,&type,&fmt,&nitems,
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == prop6 &&
+        e.xselection.property == prop6 ){
+      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop6,
+                              0,8192,True,AnyPropertyType,&type,&fmt,&nitems,
+                              &left,(WSCuchar**)&buf) == Success &&
+          nitems > 0){
+        char buffer [8193];
+        long buf2 [8193];
+        strncpy(buffer,buf,nitems);
+        if (nitems < 8193){
+          buffer[nitems] = 0;
+        }else{
+          buffer[8192] = 0;
+        }
+        WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer,WS_EN_UTF8);
+      }
+      WSGIxkeyboard()->selectionDataAvailable();
+    }else
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == prop5 &&
+//        e.xselection.property == XA_STRING ){
+        e.xselection.property == prop5){
+//printf("UTF8_STRING receive..\n");
+      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop5,
+                              0,8192,False,AnyPropertyType,&type,&fmt,&nitems,
                               &left,(WSCuchar**)&buf) == Success &&
           nitems > 0){
         char buffer [8193];
@@ -666,15 +827,150 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
         }else{
           buffer[8192] = 0;
         }
-        WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer,
-          WSGIappLocaleSet()->getSystemLocaleEncoding());
+//printf("UTF8_STRING receive.. buffer=#%s#\n",buffer);
+        WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer,WS_EN_UTF8);
+      }
+      WSGIxkeyboard()->selectionDataAvailable();
+    }else
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == prop &&
+        e.xselection.property == prop ){
+
+//printf("XA_TEXT\n");
+      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop,
+                              0,8192,False,AnyPropertyType,&type,&fmt,&nitems,
+                              &left,(WSCuchar**)&buf) == Success &&
+          nitems > 0){
+        char buffer [8193];
+        strncpy(buffer,buf,nitems);
+//printf("type=0x%x fmt=%d\n",type,fmt);
+        if (nitems < 8193){
+          buffer[nitems] = 0;
+        }else{
+          buffer[8192] = 0;
+        }
+//printf("xappdev:: comming data by cut buffer...CTEXT #%s#\n",buffer);
+//printf("#%s# nitems=%d\n",buffer,nitems);
+#if 0
+        {
+          WSCbool need_convert = False;
+          WSCushort* buffer2 = new WSCushort[nitems];
+          memset(buffer2,0,nitems*sizeof(WSCushort));
+          int cnt=0;
+          int i;
+          if (nitems > 7){
+            for(i=0; i<nitems-3;i++){
+              if (buffer[i] != 0 && buffer[i] == '\\' &&
+                  buffer[i+1] != 0 && buffer[i+1] == 'x' &&
+                  buffer[i+2] != 0 && buffer[i+2] == '{' &&
+                  buffer[i+3] != 0 &&
+                  buffer[i+4] != 0 &&
+                  buffer[i+5] != 0 &&
+                  buffer[i+6] != 0 &&
+                  buffer[i+7] != 0 && buffer[i+7] == '}'){
+                need_convert = True;
+                WSCushort val = hex_to_char(&(buffer[i+3]));
+                buffer2[cnt] = val << 8;
+                val = hex_to_char(&(buffer[i+5]));
+                buffer2[cnt] |= val;
+                cnt++;
+                i +=7;
+              }else{
+                buffer2[cnt++] = buffer[i];
+              }
+            }
+          }
+          if (need_convert != False){
+            char* buf = WSGFgetString(buffer2,
+                 WSGIappLocaleSet()->getSystemLocaleEncoding());
+            strcpy(buffer,buf);
+            delete buf;
+          }
+          delete buffer2;
+        }
+#endif
+//printf("0x%x 0x%x 0x%x 0x%x nitems=%d\n",buffer[0],buffer[1],buffer[2],buffer[3],nitems);
+        long encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+        WSCstring tmp(buffer, encoding);
+        char b1[5];
+        b1[0] = 0x1b; 
+        b1[1] = 0x24; 
+        b1[2] = 0x29; 
+        b1[3] = 0x42; 
+        b1[4] = 0; 
+        tmp.replaceString(b1,"",0);
+        b1[0] = 0x1b; 
+        b1[1] = 0x29; 
+        b1[2] = 0x49; 
+        b1[3] = 0; 
+        tmp.replaceString(b1,"",0);
+        b1[0] = 0x1b; 
+        b1[1] = 0x28; 
+        b1[2] = 0x42; 
+        b1[3] = 0; 
+        tmp.replaceString(b1,"",0);
+ 
+        WSGIxkeyboard()->WSDkeyboard::setSelectedString(
+                                    tmp.getString(encoding),encoding);
 //printf("type=%d fmt=%d left=%d\n",type,fmt,left);
-//printf("xappdev:: comming data by cut buffer...#%s# %d\n",buffer, WSGIappLocaleSet()->getSystemLocaleEncoding());
+//printf("xappdev:: comming data by cut buffer...#%s# strlen=%d %d\n",buffer,strlen(buffer), WSGIappLocaleSet()->getSystemLocaleEncoding());
 //        WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer);
 //printf("xappdev:: comming data3...type=%d %d nitems=%d #%s#\n",type,fmt,nitems,buf);
       }else{
 //printf("xappdev:: comming data... none...\n");
       }
+      WSGIxkeyboard()->selectionDataAvailable();
+    }else
+    if (e.xselection.selection == XA_PRIMARY &&
+        e.xselection.target == prop2 &&
+        e.xselection.property == prop2 ){
+      if (XGetWindowProperty(_disp,XtWindow(appWidget()),prop2,
+                              0,8192,True,AnyPropertyType,&type,&fmt,&nitems,
+                              &left,(WSCuchar**)&buf) == Success &&
+          nitems > 0){
+        char buffer [8193];
+        long buf2 [8193];
+        strncpy(buffer,buf,nitems);
+        if (nitems < 8193){
+          buffer[nitems] = 0;
+        }else{
+          buffer[8192] = 0;
+        }
+        long encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+        WSCstring tmp(buffer, encoding);
+        char b1[5];
+        b1[0] = 0x1b; 
+        b1[1] = 0x24; 
+        b1[2] = 0x29; 
+        b1[3] = 0x42; 
+        b1[4] = 0; 
+        tmp.replaceString(b1,"",0);
+        b1[0] = 0x1b; 
+        b1[1] = 0x24; 
+        b1[2] = 0x28; 
+        b1[3] = 0x42; 
+        b1[4] = 0; 
+        if (tmp.isExist(b1) != -1){
+          char* str = WSGFcreateEucStringFromJisStr(buffer);
+          tmp.setString(str,WS_EN_EUCJP);
+          delete str;
+        }
+
+        b1[0] = 0x1b; 
+        b1[1] = 0x29; 
+        b1[2] = 0x49; 
+        b1[3] = 0; 
+        tmp.replaceString(b1,"",0);
+        b1[0] = 0x1b; 
+        b1[1] = 0x28; 
+        b1[2] = 0x42; 
+        b1[3] = 0; 
+        tmp.replaceString(b1,"",0);
+//printf("0x%x 0x%x 0x%x 0x%x nitems=%d\n",buffer[0],buffer[1],buffer[2],buffer[3],nitems);
+        WSGIxkeyboard()->WSDkeyboard::setSelectedString(
+                                    tmp.getString(encoding),encoding);
+      }
+      WSGIxkeyboard()->selectionDataAvailable();
     }else{
       int bytes;
       WSDxappDev* app   = WSGIxwinAppDev();
@@ -682,12 +978,12 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
       char* str = new char[bytes+1];
       memcpy(str,ret,bytes);
       str[bytes]=0;
+//printf("xappdev:: comming data by cut buffer...#%s# %d\n",str, WSGIappLocaleSet()->getSystemLocaleEncoding());
       WSGIxkeyboard()->WSDkeyboard::setSelectedString(str,
          WSGIappLocaleSet()->getSystemLocaleEncoding());
       delete str;
-//printf("xappdev:: comming data by cut buffer...#%s# %d\n",str, WSGIappLocaleSet()->getSystemLocaleEncoding());
+      WSGIxkeyboard()->selectionDataAvailable();
     }
-    WSGIxkeyboard()->selectionDataAvailable();
   }
 
   if (e.type == KeyPress || e.type == KeyRelease || e.type == FocusIn){
