@@ -707,7 +707,12 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
         tmp = buf;
         tmp_mask = buf_mask;
         for (y = 0; y < height; y++){
-          ftmp = fbuf + ((height - 1 - y) * width * 3);
+          if (cmptype != 3){
+//            ftmp = fbuf + (height - 1 - y) * width * 3;
+            ftmp = fbuf + (height - 1 - y) * (((width * 3)+3)/4)*4;
+          }else{
+            ftmp = fbuf + ((height - 1 - y) * width * 4);
+          }
           for (x = 0; x < width; x++){
             *(tmp++) = *(ftmp + 2);       // red
             *(tmp++) = *(ftmp + 1);       // green
@@ -715,6 +720,7 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
             if (cmptype != 3){
               ftmp += 3;
             }else{
+              *(tmp_mask++) = *(ftmp + 3);
               ftmp += 4;
             }
           }
