@@ -900,6 +900,7 @@ void WSDxkeyboard::setSelectedString(char* str,long encode){
 
   WSCstring string;
   string.setString(str,encode);
+
   XStoreBytes(app->display(),
               string.getString(WSGIappLocaleSet()->getSystemLocaleEncoding()),
     strlen( string.getString(WSGIappLocaleSet()->getSystemLocaleEncoding()))+1);
@@ -933,10 +934,13 @@ static long _lock = 0;
     WSDxappDev* app   = WSGIxwinAppDev(); 
 static Atom prop = 0;
     if (prop == 0){
-      prop = XInternAtom(app->display(),"COMPOUND_TEXT",False);
+//      prop = XInternAtom(app->display(),"COMPOUND_TEXT",False);
+//      prop = XInternAtom(app->display(),"UTF8_STRING",False);
+      prop = XInternAtom(app->display(),"TARGETS",False);
     }
     Window me = XtWindow(app->appWidget());
-    XConvertSelection(app->display(),XA_PRIMARY,XA_STRING,prop,me,CurrentTime);
+//    XConvertSelection(app->display(),XA_PRIMARY,XA_STRING,prop,me,CurrentTime);
+    XConvertSelection(app->display(),XA_PRIMARY,prop,prop,me,CurrentTime);
 //printf("WSDxkeyboard::getSelectedString... create request...\n");
 
 
