@@ -41,6 +41,7 @@
 #include <errno.h>
 #include <sys/file.h>
 #include <string.h>
+#include <x11/WSxcom.h>
 
 #ifdef SUN
 #define SOCKLEN_T  int
@@ -69,11 +70,11 @@ class  _xsocket_init {
 };
 _xsocket_init   WSGIappXsocketInit;
 
-typedef struct pthread_sem_struct{
-  int locked;
-  pthread_mutex_t mutex;
-  pthread_cond_t lock_free;
-} pthread_sem_t;
+//typedef struct pthread_sem_struct{
+//  int locked;
+//  pthread_mutex_t mutex;
+//  pthread_cond_t lock_free;
+//} pthread_sem_t;
 
 // A new semaphore function for initializing.
 int pthread_sem_init_np(pthread_sem_t* semp){
@@ -348,6 +349,7 @@ void* WSDxsocket::_udp_read_thread_(void* ptr){
     server.sin_port = htons( _this->_port );
 //printf("WSDxsocket::_udp_read_thread bind port=%d server.sin_port=%d\n",_this->_port,server.sin_port);
     if( bind(_this->_socket, (struct sockaddr *)&server, sizeof(server)) < 0){
+      XCloseDisplay(disp);
       return NULL;
     }
     _this->_udp_binded = True;
@@ -429,6 +431,7 @@ void* WSDxsocket::_udp_read_thread_(void* ptr){
       break;
     }
   }
+  XCloseDisplay(disp);
   return 0;
 }
 
@@ -597,19 +600,19 @@ long WSDxsocket::read(void* handle,WSCuchar* buf,long size){
         read_len +=  ret;
         len_to_read -= ret;
         if (len_to_read == 0){
-//printf("WSDxsocket::read... here7\n");
+//printf("WSDxsocket::read... here7 read_len=%d\n",read_len);
           seterr();
           return read_len;
         }
       }else if (ret == 0){        
         if (read_first == False){
-//printf("WSDxsocket::read... here6\n");
+//printf("WSDxsocket::read... here6 read_len=%d\n",read_len);
           seterr();
           return read_len;
         }
       }else if (ret < 0){        
         if (errno != EAGAIN){
-//printf("WSDxsocket::read... here5\n");
+//printf("WSDxsocket::read... here5 read_len=%d\n",read_len);
           seterr();
           return read_len;
         }
