@@ -971,7 +971,8 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
                   proxy_port==80 ? "" : ":", proxy_port==80 ? "" : port_str);
       }
     }else if (status == WS_NO_ERR){
-      if (WSGFhttpWrite(sock, (*_http_proxy=='\0' ? path : url)) == WS_NO_ERR) {
+//      if (WSGFhttpWrite(sock, (*_http_proxy=='\0' ? path : url)) == WS_NO_ERR) {
+      if (WSGFhttpWrite(sock,url) == WS_NO_ERR) {
         if ((status=WSGFhttpRead(sock, &buf, &size)) ==
                 WS_NO_ERR && buf != NULL && *buf != '\0') {
                *pbuffer = _http_extxt(buf, bsize, fl,ctype);
